"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = void 0;
const decode_1 = require("../../../util/base64/decode");
const clock_1 = require("../../clock");
const PatchBuilder_1 = require("../../PatchBuilder");
const decode = (data) => {
    const x = data[0];
    const clock = Array.isArray(x) ? new clock_1.VectorClock(x[0], x[1]) : new clock_1.ServerVectorClock(1, x);
    const sessionId = clock.sid;
    const time = clock.time;
    const builder = new PatchBuilder_1.PatchBuilder(clock);
    const length = data.length;
    let i = 1;
    const decodeTimestamp = () => {
        const x = data[i++];
        if (Array.isArray(x))
            return (0, clock_1.ts)(x[0], x[1]);
        else if (x < 0)
            return (0, clock_1.ts)(sessionId, time - x - 1);
        else
            return (0, clock_1.ts)(1, x);
    };
    const decodeTimespan = () => {
        const time = decodeTimestamp();
        const span = data[i++];
        return (0, clock_1.interval)(time, 0, span);
    };
    while (i < length) {
        switch (data[i++]) {
            case 3: {
                builder.obj();
                break;
            }
            case 5: {
                builder.arr();
                break;
            }
            case 4: {
                builder.str();
                break;
            }
            case 6: {
                builder.bin();
                break;
            }
            case 9: {
                const length = data[i++];
                const obj = decodeTimestamp();
                const tuples = [];
                for (let j = 0; j < length; j++) {
                    const key = data[i++];
                    tuples.push([key, decodeTimestamp()]);
                }
                builder.setKeys(obj, tuples);
                break;
            }
            case 7: {
                const value = data[i++];
                builder.insStr(decodeTimestamp(), decodeTimestamp(), value);
                break;
            }
            case 15: {
                const value = data[i++];
                builder.insBin(decodeTimestamp(), decodeTimestamp(), (0, decode_1.fromBase64)(value));
                break;
            }
            case 11: {
                const length = data[i++];
                const arr = decodeTimestamp();
                const after = decodeTimestamp();
                const values = [];
                for (let j = 0; j < length; j++)
                    values.push(decodeTimestamp());
                builder.insArr(arr, after, values);
                break;
            }
            case 12: {
                const obj = decodeTimestamp();
                const what = decodeTimespan();
                builder.del(obj, [what]);
                break;
            }
            case 8: {
                const length = data[i++];
                const obj = decodeTimestamp();
                const what = [];
                for (let i = 0; i < length; i++)
                    what.push(decodeTimespan());
                builder.del(obj, what);
                break;
            }
            case 13: {
                builder.noop(1);
                break;
            }
            case 14: {
                builder.noop(data[i++]);
                break;
            }
            case 1: {
                builder.const(data[i++]);
                break;
            }
            case 0: {
                builder.const(undefined);
                break;
            }
            case 2: {
                builder.val(decodeTimestamp());
                break;
            }
            case 10: {
                const obj = decodeTimestamp();
                const value = decodeTimestamp();
                builder.setVal(obj, value);
                break;
            }
        }
    }
    return builder.patch;
};
exports.decode = decode;
