"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Finder = void 0;
const ArrayApi_1 = require("./ArrayApi");
const ArrayRga_1 = require("../../types/rga-array/ArrayRga");
const BinaryApi_1 = require("./BinaryApi");
const BinaryRga_1 = require("../../types/rga-binary/BinaryRga");
const ObjectApi_1 = require("./ObjectApi");
const ObjectLww_1 = require("../../types/lww-object/ObjectLww");
const json_pointer_1 = require("../../../json-pointer");
const StringApi_1 = require("./StringApi");
const StringRga_1 = require("../../types/rga-string/StringRga");
const ValueApi_1 = require("./ValueApi");
const ValueLww_1 = require("../../types/lww-value/ValueLww");
const Const_1 = require("../../types/const/Const");
const ConstApi_1 = require("./ConstApi");
class Finder {
    constructor(node, api) {
        this.node = node;
        this.api = api;
    }
    find(path) {
        const steps = (0, json_pointer_1.toPath)(path);
        let node = this.node;
        const length = steps.length;
        if (!length)
            return node;
        let i = 0;
        while (i < length && node) {
            const step = steps[i++];
            node = node.container();
            if (!node)
                throw new Error('NOT_CONTAINER');
            if (node instanceof ObjectLww_1.ObjectLww) {
                const nextNode = node.get(String(step));
                if (!nextNode)
                    throw new Error('NOT_FOUND');
                node = nextNode;
            }
            else if (node instanceof ArrayRga_1.ArrayRga) {
                const nextNode = node.getNode(Number(step));
                if (!nextNode)
                    throw new Error('NOT_FOUND');
                node = nextNode;
            }
        }
        return node;
    }
    val(path) {
        const node = this.find(path);
        if (node instanceof ValueLww_1.ValueLww)
            return new ValueApi_1.ValueApi(this.api, node);
        throw new Error('NOT_VAL');
    }
    str(path) {
        const node = this.find(path);
        if (node instanceof StringRga_1.StringRga)
            return new StringApi_1.StringApi(this.api, node);
        throw new Error('NOT_STR');
    }
    bin(path) {
        const node = this.find(path);
        if (node instanceof BinaryRga_1.BinaryRga)
            return new BinaryApi_1.BinaryApi(this.api, node);
        throw new Error('NOT_BIN');
    }
    arr(path) {
        const node = this.find(path);
        if (node instanceof ArrayRga_1.ArrayRga)
            return new ArrayApi_1.ArrayApi(this.api, node);
        throw new Error('NOT_ARR');
    }
    obj(path) {
        const obj = this.find(path);
        if (obj instanceof ObjectLww_1.ObjectLww)
            return new ObjectApi_1.ObjectApi(this.api, obj);
        throw new Error('NOT_OBJ');
    }
    const(path) {
        const obj = this.find(path);
        if (obj instanceof Const_1.Const)
            return new ConstApi_1.ConstApi(obj);
        throw new Error('NOT_CONST');
    }
}
exports.Finder = Finder;
