"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelApi = void 0;
const Finder_1 = require("./Finder");
const PatchBuilder_1 = require("../../../json-crdt-patch/PatchBuilder");
class ModelApi {
    constructor(model) {
        this.model = model;
        this.next = 0;
        this.builder = new PatchBuilder_1.PatchBuilder(this.model.clock);
    }
    find(path) {
        const finder = new Finder_1.Finder(this.getRoot(), this);
        return path ? finder.find(path) : finder;
    }
    val(path) {
        return this.find().val(path);
    }
    str(path) {
        return this.find().str(path);
    }
    bin(path) {
        return this.find().bin(path);
    }
    arr(path) {
        return this.find().arr(path);
    }
    obj(path) {
        return this.find().obj(path);
    }
    const(path) {
        return this.find().const(path);
    }
    root(json) {
        const builder = this.builder;
        builder.root(builder.json(json));
        this.apply();
        return this;
    }
    getRoot() {
        return this.model.root.node();
    }
    apply() {
        const ops = this.builder.patch.ops;
        const length = ops.length;
        for (let i = this.next; i < length; i++)
            this.model.applyOperation(ops[i]);
        this.next = length;
    }
    advance() {
        this.next = this.builder.patch.ops.length;
    }
    view() {
        return this.model.view();
    }
    flush() {
        const patch = this.builder.flush();
        this.next = 0;
        return patch;
    }
}
exports.ModelApi = ModelApi;
