"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectLww = void 0;
const clock_1 = require("../../../json-crdt-patch/clock");
class ObjectLww {
    constructor(doc, id) {
        this.doc = doc;
        this.id = id;
        this.latest = new Map();
        this._view = {};
    }
    get(key) {
        const id = this.latest.get(key);
        if (!id)
            return undefined;
        return this.doc.index.get(id);
    }
    getId(key) {
        return this.latest.get(key);
    }
    put(key, id) {
        const currentId = this.latest.get(key);
        if (currentId && (0, clock_1.compare)(currentId, id) >= 0)
            return;
        this.latest.set(key, id);
        return currentId;
    }
    view() {
        const _toJson = this._view;
        const obj = {};
        let useCache = true;
        const index = this.doc.index;
        for (const [key, id] of this.latest.entries()) {
            const value = index.get(id).view();
            if (value !== undefined) {
                if (_toJson[key] !== value)
                    useCache = false;
                obj[key] = value;
            }
            else {
                if (_toJson[key] !== undefined)
                    useCache = false;
            }
        }
        return useCache ? _toJson : (this._view = obj);
    }
    toString(tab = '') {
        let str = `${this.constructor.name} ${(0, clock_1.toDisplayString)(this.id)}`;
        const length = this.latest.size;
        let cnt = 0;
        for (const [key, id] of this.latest.entries()) {
            cnt++;
            const isLast = cnt === length;
            const leftTab = isLast ? '  ' : '│ ';
            str += `\n${tab}${isLast ? '└─' : '├─'} ${JSON.stringify(key)}`;
            str += `\n${tab}${leftTab}  └─ ${this.doc.index.get(id).toString(tab + leftTab + '     ')}`;
        }
        return str;
    }
    children(callback) {
        const index = this.doc.index;
        this.latest.forEach((id, key) => callback(index.get(id)));
    }
    nodes(callback) {
        const index = this.doc.index;
        this.latest.forEach((id, key) => callback(index.get(id), key));
    }
    container() {
        return this;
    }
}
exports.ObjectLww = ObjectLww;
