import { ITimestampStruct, ITimespanStruct } from '../../../json-crdt-patch/clock';
import { Tree } from '../../../util/trees/Tree';
export interface Chunk<T> {
    id: ITimestampStruct;
    span: number;
    del: boolean;
    data: T | undefined;
    len: number;
    p: Chunk<T> | undefined;
    l: Chunk<T> | undefined;
    r: Chunk<T> | undefined;
    s: Chunk<T> | undefined;
    merge(content: T): void;
    split(ticks: number): Chunk<T>;
    delete(): void;
    clone(): Chunk<T>;
}
export declare abstract class AbstractRga<T> {
    readonly id: ITimestampStruct;
    root: Chunk<T> | undefined;
    ids: Tree<ITimestampStruct, Chunk<T>>;
    abstract view(): unknown;
    protected abstract createChunk(id: ITimestampStruct, content: T | undefined): Chunk<T>;
    protected abstract onViewChange(): void;
    constructor(id: ITimestampStruct);
    ins(ref: ITimestampStruct, id: ITimestampStruct, content: T): void;
    insAt(position: number, id: ITimestampStruct, content: T): ITimestampStruct | undefined;
    delete(spans: ITimespanStruct[]): void;
    protected deleteSpan(span: ITimespanStruct): void;
    find(position: number): void | ITimestampStruct;
    findChunk(position: number): void | [chunk: Chunk<T>, offset: number];
    findInterval(position: number, length: number): ITimespanStruct[];
    first(): Chunk<T> | undefined;
    next(curr: Chunk<T>): Chunk<T> | undefined;
    prev(curr: Chunk<T>): Chunk<T> | undefined;
    length(): number;
    size(): number;
    pos(chunk: Chunk<T>): number;
    setRoot(chunk: Chunk<T>): void;
    insertBefore(chunk: Chunk<T>, before: Chunk<T>): void;
    insertAfter(chunk: Chunk<T>, after: Chunk<T>): void;
    protected insertAfterRef(chunk: Chunk<T>, ref: ITimestampStruct, left: Chunk<T>): void;
    protected mergeContent(chunk: Chunk<T>, content: T): void;
    protected insertInside(chunk: Chunk<T>, at: Chunk<T>, offset: number): void;
    protected split(chunk: Chunk<T>, ticks: number): Chunk<T>;
    protected mergeTombstones(ch1: Chunk<T>, ch2: Chunk<T>): boolean;
    protected mergeTombstones2(start: Chunk<T>, end: Chunk<T>): void;
    deleteChunk(chunk: Chunk<T>): void;
    insertId(chunk: Chunk<T>): void;
    insertIdFast(chunk: Chunk<T>): void;
    protected deleteId(chunk: Chunk<T>): void;
    findById(id: ITimestampStruct): undefined | Chunk<T>;
    protected dLen(chunk: Chunk<T> | undefined, delta: number): void;
    protected updateLenOne(chunk: Chunk<T>): void;
    splay(chunk: Chunk<T>, repeat: number): void;
    toString(tab?: string): string;
    protected toStringChunk(chunk: Chunk<T>, tab: string, side: 'l' | 'r' | ''): string;
    protected formatChunk(chunk: Chunk<T>, tab: string, side: 'l' | 'r' | ''): string;
    iterator(): () => Chunk<T> | undefined;
    ingest(size: number, next: () => Chunk<T>): void;
    private _ingest;
}
