"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkerPool = void 0;
const tslib_1 = require("tslib");
const Defer_1 = require("../Defer");
const worker_threads_1 = require("worker_threads");
const WorkerPoolModule_1 = require("./WorkerPoolModule");
const path = tslib_1.__importStar(require("path"));
class WorkerPool {
    constructor() {
        this.seq = 0;
        this.nextWorker = 0;
        this.workers = [];
        this.requests = new Map();
        this.modules = new Map();
    }
    queue() {
        return this.requests.size;
    }
    async addThread() {
        const fileName = path.join(__dirname, 'worker', 'main');
        const worker = new worker_threads_1.Worker(fileName);
        await new Promise((resolve) => {
            worker.once('message', (msg) => resolve());
        });
        worker.on('message', (msg) => {
            if (msg instanceof Array)
                this.onResponse(msg);
        });
        for (const module of this.modules.values())
            await module.setupWorker(worker);
        this.workers.push(worker);
    }
    async addThreads(count) {
        const promises = [];
        for (let i = 0; i < count; i++)
            promises.push(this.addThread());
        await Promise.all(promises);
    }
    async load(name, file) {
        if (this.modules.has(name))
            throw new Error(`Duplicate [module = ${name}].`);
        const module = new WorkerPoolModule_1.WorkerPoolModule(this, name, file);
        this.modules.set(name, module);
        await Promise.all(this.workers.map((worker) => module.setupWorker(worker)));
        return module;
    }
    exec(id, data, transferList, worker = this.pickWorker()) {
        const seq = this.seq++;
        const future = new Defer_1.Defer();
        const request = [seq, id, data];
        this.requests.set(seq, future);
        try {
            worker.postMessage(request, transferList);
            return future.promise;
        }
        catch {
            this.requests.delete(seq);
            return Promise.reject(new Error('PORT'));
        }
    }
    onResponse([seq, data, isError]) {
        const requests = this.requests;
        const future = requests.get(seq);
        if (!future)
            return;
        requests.delete(seq);
        if (isError)
            future.reject(data);
        else
            future.resolve(data);
    }
    pickWorker() {
        const worker = this.workers[this.nextWorker];
        this.nextWorker = (this.nextWorker + 1) % this.workers.length;
        return worker;
    }
}
exports.WorkerPool = WorkerPool;
