"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
const ArrInsOp_1 = require("../../operations/ArrInsOp");
const ArrOp_1 = require("../../operations/ArrOp");
const BinInsOp_1 = require("../../operations/BinInsOp");
const BinOp_1 = require("../../operations/BinOp");
const CrdtEncoder_1 = require("../../util/binary/CrdtEncoder");
const DelOp_1 = require("../../operations/DelOp");
const NoopOp_1 = require("../../operations/NoopOp");
const ObjOp_1 = require("../../operations/ObjOp");
const ObjSetOp_1 = require("../../operations/ObjSetOp");
const StrInsOp_1 = require("../../operations/StrInsOp");
const StrOp_1 = require("../../operations/StrOp");
const ValOp_1 = require("../../operations/ValOp");
const ValSetOp_1 = require("../../operations/ValSetOp");
const ConstOp_1 = require("../../operations/ConstOp");
const MsgPackEncoder_1 = require("../../../json-pack/msgpack/MsgPackEncoder");
class Encoder extends MsgPackEncoder_1.MsgPackEncoder {
    constructor() {
        super(new CrdtEncoder_1.CrdtWriter());
    }
    encode(patch) {
        this.writer.reset();
        const id = (this.patchId = patch.getId());
        const isServerClock = id.sid === 1;
        if (isServerClock) {
            this.writer.b1vu56(true, id.time);
        }
        else {
            this.writer.b1vu56(false, id.sid);
            this.writer.vu57(id.time);
        }
        this.encodeOperations(patch);
        return this.writer.flush();
    }
    encodeOperations(patch) {
        const ops = patch.ops;
        for (let i = 0; i < ops.length; i++) {
            const op = ops[i];
            this.encodeOperation(op);
        }
    }
    encodeId(id) {
        const sessionId = id.sid;
        const time = id.time;
        if (sessionId === 1) {
            this.writer.b1vu56(true, id.time);
        }
        else {
            const patchId = this.patchId;
            if (sessionId === patchId.sid && time >= patchId.time) {
                this.writer.b1vu56(false, 1);
                this.writer.vu57(time - patchId.time);
            }
            else {
                this.writer.b1vu56(false, sessionId);
                this.writer.vu57(time);
            }
        }
    }
    encodeTss(span) {
        this.encodeId(span);
        this.writer.vu57(span.span);
    }
    encodeOperation(op) {
        if (op instanceof ObjOp_1.ObjOp)
            this.writer.u8(3);
        else if (op instanceof ArrOp_1.ArrOp)
            this.writer.u8(5);
        else if (op instanceof StrOp_1.StrOp)
            this.writer.u8(4);
        else if (op instanceof ConstOp_1.ConstOp) {
            const val = op.val;
            if (val === undefined) {
                this.writer.u8(0);
            }
            else {
                this.writer.u8(1);
                this.encodeAny(op.val);
            }
        }
        else if (op instanceof ValOp_1.ValOp) {
            this.writer.u8(2);
            this.encodeId(op.val);
        }
        else if (op instanceof ObjSetOp_1.ObjSetOp) {
            this.writer.u8(9);
            this.encodeId(op.obj);
            this.writer.vu57(op.data.length);
            for (const [key, value] of op.data) {
                this.encodeId(value);
                this.encodeString(key);
            }
        }
        else if (op instanceof ValSetOp_1.ValSetOp) {
            this.writer.u8(10);
            this.encodeId(op.obj);
            this.encodeId(op.val);
        }
        else if (op instanceof StrInsOp_1.StrInsOp) {
            this.writer.u8(7);
            this.encodeId(op.obj);
            this.encodeId(op.ref);
            this.encodeString(op.data);
        }
        else if (op instanceof ArrInsOp_1.ArrInsOp) {
            const { obj: arr, ref: after, data: elements } = op;
            const length = elements.length;
            this.writer.u8(11);
            this.encodeId(arr);
            this.encodeId(after);
            this.writer.vu57(length);
            for (let i = 0; i < length; i++)
                this.encodeId(elements[i]);
        }
        else if (op instanceof DelOp_1.DelOp) {
            const { obj, what } = op;
            const length = what.length;
            if (length > 1) {
                this.writer.u8(8);
                this.encodeId(obj);
                this.writer.vu57(length);
                for (let i = 0; i < length; i++)
                    this.encodeTss(what[i]);
            }
            else {
                this.writer.u8(12);
                this.encodeId(obj);
                this.encodeTss(what[0]);
            }
        }
        else if (op instanceof NoopOp_1.NoopOp) {
            const { len: length } = op;
            if (length > 1) {
                this.writer.u8(14);
                this.writer.vu57(length);
            }
            else
                this.writer.u8(13);
        }
        else if (op instanceof BinOp_1.BinOp)
            this.writer.u8(6);
        else if (op instanceof BinInsOp_1.BinInsOp) {
            const buf = op.data;
            const length = buf.length;
            this.writer.u8(15);
            this.encodeId(op.obj);
            this.encodeId(op.ref);
            this.writer.vu57(length);
            this.writer.buf(buf, length);
        }
        else
            throw new Error('UNKNOWN_OP');
    }
}
exports.Encoder = Encoder;
