"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkerPoolModule = void 0;
const tslib_1 = require("tslib");
const WorkerResponse_1 = require("./WorkerResponse");
class WorkerPoolModule {
    constructor(pool, name, file) {
        this.pool = pool;
        this.name = name;
        this.file = file;
        this.toId = new Map();
        this.type = new Map();
    }
    setupWorker(worker) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const msg = {
                type: 'load',
                file: this.file,
            };
            worker.postMessage(msg);
            const { list } = yield new Promise((resolve) => {
                const listener = (msg) => {
                    if (typeof msg === 'object' && msg.type === 'loaded') {
                        worker.removeListener('message', listener);
                        resolve(msg);
                    }
                };
                worker.addListener('message', listener);
            });
            for (const [id, name, type] of list) {
                this.toId.set(name, id);
                this.type.set(name, type === 'ch' ? 1 : 0);
            }
        });
    }
    fn(name, worker) {
        const id = this.toId.get(name);
        if (id === undefined)
            throw new Error('UNKNOWN_FN');
        return ((req, transferList) => tslib_1.__awaiter(this, void 0, void 0, function* () { return yield this.pool.exec(id, req, transferList, worker); }));
    }
    ch(name, worker) {
        const id = this.toId.get(name);
        if (id === undefined)
            throw new Error('UNKNOWN_CH');
        return ((req, transferList) => {
            worker = worker || this.pool.pickWorker();
            let cb;
            let closed = false;
            const seq = this.pool.seq;
            const response = this.pool.exec(id, req, transferList, worker);
            const take = (callback) => {
                if (closed)
                    throw new Error('CHANNEL_CLOSED');
                cb = callback;
            };
            const send = (data) => {
                if (closed)
                    throw new Error('CHANNEL_CLOSED');
                let transferList;
                if (data instanceof WorkerResponse_1.WorkerResponse) {
                    transferList = data.transferList;
                    data = data.data;
                }
                const msg = [[seq], data];
                worker.postMessage(msg, transferList);
            };
            const listener = (msg) => {
                if (msg instanceof Array) {
                    if (msg[0] instanceof Array) {
                        if (msg[0][0] === seq) {
                            if (cb)
                                cb(msg[1]);
                        }
                    }
                }
            };
            worker.addListener('message', listener);
            return [
                take,
                send,
                response.finally(() => {
                    closed = true;
                    worker.removeListener('message', listener);
                }),
            ];
        });
    }
    api(worker) {
        const methods = {};
        for (const name of this.toId.keys()) {
            const isChannel = this.type.get(name) === 1;
            methods[name] = isChannel ? this.ch(name, worker) : this.fn(name, worker);
        }
        return methods;
    }
}
exports.WorkerPoolModule = WorkerPoolModule;
