/// <reference types="node" />
import type { TransferListItem } from 'worker_threads';
import type { WorkerChannel } from './WorkerChannel';
import type { WorkerResponse } from './WorkerResponse';
export declare type TransferList = ReadonlyArray<TransferListItem>;
export declare type MaybePromise<T> = T | Promise<T>;
export declare type WorkerFn<Request = unknown, Response = unknown> = (req: Request) => MaybePromise<Response | WorkerResponse<Response>>;
export declare type WPMsgExecRequest<T = unknown> = [seq: number, method: number, data: T];
export declare type WPMsgExecResponse<T = unknown> = [seq: number, data: T];
export declare type WPMsgExecErrorResponse<T = unknown> = [seq: number, data: T, isError: true];
export declare type WPMsgChannel<T = unknown> = [seq: [number], data: T];
export interface WPMsgWorkerReady {
    type: 'ready';
}
export interface WPMsgLoad {
    type: 'load';
    file: string;
}
export interface WPMsgLoaded {
    type: 'loaded';
    list: Array<[id: number, name: string, type: 'fn' | 'ch']>;
}
export declare type WPRpcMessage = WPMsgExecRequest | WPMsgExecResponse | WPMsgExecErrorResponse | WPMsgChannel;
export declare type WPControlMessage = WPMsgWorkerReady | WPMsgLoad | WPMsgLoaded;
export declare type WPMessage = WPRpcMessage | WPControlMessage;
export declare type WorkerMethod<Request, Response> = Request extends void ? () => Promise<Response> : (request: Request, transferList?: TransferList | undefined) => Promise<Response>;
export declare type WorkerMethodCh<Request, Incoming, Outgoing, Response> = Request extends void ? () => [take: WorkerTake<Outgoing>, send: WorkerSend<Incoming>, response: Promise<Response>] : (request: Request, transferList?: TransferList | undefined) => [take: WorkerTake<Outgoing>, send: WorkerSend<Incoming>, response: Promise<Response>];
export declare type WorkerMethods<T> = {
    [K in keyof T]: T[K] extends WorkerFn<infer Request, infer Response> ? WorkerMethod<Request, Response> : T[K] extends WorkerChannel<infer Request, infer Incoming, infer Outgoing, infer Response> ? WorkerMethodCh<Request, Incoming, Outgoing, Response> : WorkerMethod<void, T[K]>;
};
export declare type WorkerSend<Msg> = (data: Msg | WorkerResponse<Msg>) => void;
export declare type WorkerTake<Msg> = (callback: (data: Msg) => void) => void;
export declare type WorkerCh<Request, Incoming, Outgoing, Response> = (req: Request, take: WorkerTake<Incoming>, send: WorkerSend<Outgoing>) => Promise<Response | WorkerResponse<Response>>;
