import { CrdtDecoder } from '../../util/binary/CrdtDecoder';
import { interval, VectorClock, ServerVectorClock, ts } from '../../clock';
import { PatchBuilder } from '../../PatchBuilder';
import { MsgPackDecoderFast } from '../../../json-pack/msgpack';
export class Decoder extends MsgPackDecoderFast {
    builder;
    patchId;
    constructor() {
        super(new CrdtDecoder());
    }
    decode(data) {
        this.reader.reset(data);
        const [isServerClock, x] = this.reader.b1vu56();
        const clock = isServerClock ? new ServerVectorClock(1, x) : new VectorClock(x, this.reader.vu57());
        this.patchId = ts(clock.sid, clock.time);
        this.builder = new PatchBuilder(clock);
        this.decodeOperations();
        return this.builder.patch;
    }
    decodeId() {
        const [isServerClock, x] = this.reader.b1vu56();
        if (isServerClock) {
            return ts(1, x);
        }
        else {
            const patchId = this.patchId;
            if (x === 1) {
                const delta = this.reader.vu57();
                return ts(patchId.sid, patchId.time + delta);
            }
            else {
                const time = this.reader.vu57();
                return ts(x, time);
            }
        }
    }
    decodeTss() {
        const id = this.decodeId();
        const span = this.reader.vu57();
        return interval(id, 0, span);
    }
    decodeOperations() {
        while (this.reader.x < this.reader.uint8.length)
            this.decodeOperation();
    }
    decodeOperation() {
        const opcode = this.reader.u8();
        switch (opcode) {
            case 3: {
                this.builder.obj();
                return;
            }
            case 5: {
                this.builder.arr();
                return;
            }
            case 4: {
                this.builder.str();
                return;
            }
            case 2: {
                this.builder.val(this.decodeId());
                return;
            }
            case 1: {
                const val = this.val();
                this.builder.const(val);
                return;
            }
            case 0: {
                this.builder.const(undefined);
                return;
            }
            case 9: {
                const object = this.decodeId();
                const fields = this.reader.vu57();
                const tuples = [];
                for (let i = 0; i < fields; i++) {
                    const value = this.decodeId();
                    const key = this.val();
                    if (typeof key !== 'string')
                        continue;
                    tuples.push([key, value]);
                }
                this.builder.setKeys(object, tuples);
                return;
            }
            case 10: {
                const obj = this.decodeId();
                const val = this.decodeId();
                this.builder.setVal(obj, val);
                return;
            }
            case 7: {
                const obj = this.decodeId();
                const after = this.decodeId();
                const str = this.val();
                if (typeof str !== 'string')
                    return;
                this.builder.insStr(obj, after, str);
                return;
            }
            case 11: {
                const arr = this.decodeId();
                const after = this.decodeId();
                const length = this.reader.vu57();
                const elements = [];
                for (let i = 0; i < length; i++)
                    elements.push(this.decodeId());
                this.builder.insArr(arr, after, elements);
                return;
            }
            case 8: {
                const obj = this.decodeId();
                const length = this.reader.vu57();
                const what = [];
                for (let i = 0; i < length; i++)
                    what.push(this.decodeTss());
                this.builder.del(obj, what);
                return;
            }
            case 12: {
                const obj = this.decodeId();
                const span = this.decodeTss();
                this.builder.del(obj, [span]);
                return;
            }
            case 13: {
                this.builder.noop(1);
                return;
            }
            case 14: {
                this.builder.noop(this.reader.vu57());
                return;
            }
            case 6: {
                this.builder.bin();
                return;
            }
            case 15: {
                const obj = this.decodeId();
                const after = this.decodeId();
                const length = this.reader.vu57();
                const data = this.bin(length);
                this.builder.insBin(obj, after, data);
                return;
            }
            default: {
                throw new Error('UNKNOWN_OP');
            }
        }
    }
}
