import { ArrayRga } from '../../../types/rga-array/ArrayRga';
import { BinaryRga } from '../../../types/rga-binary/BinaryRga';
import { ClockEncoder } from '../../../../json-crdt-patch/codec/clock/ClockEncoder';
import { Const } from '../../../types/const/Const';
import { CrdtWriter } from '../../../../json-crdt-patch/util/binary/CrdtEncoder';
import { MsgPackEncoder } from '../../../../json-pack/msgpack';
import { ObjectLww } from '../../../types/lww-object/ObjectLww';
import { StringRga } from '../../../types/rga-string/StringRga';
import { ValueLww } from '../../../types/lww-value/ValueLww';
export class Encoder extends MsgPackEncoder {
    clockEncoder = new ClockEncoder();
    time = 0;
    doc;
    constructor() {
        super(new CrdtWriter());
    }
    encode(doc) {
        this.doc = doc;
        this.writer.reset();
        if (doc.clock.sid === 1)
            this.encodeServer(doc);
        else
            this.encodeLogical(doc);
        return this.writer.flush();
    }
    encodeLogical(model) {
        const writer = this.writer;
        this.ts = this.tsLogical;
        let clockTableOffset = 0;
        writer.u8(1);
        this.clockEncoder.reset(model.clock);
        clockTableOffset = writer.x;
        writer.ensureCapacity(4);
        writer.x += 4;
        this.cRoot(model.root);
        this.encodeClockTable(clockTableOffset);
    }
    encodeServer(model) {
        this.ts = this.tsServer;
        this.writer.u8(0);
        this.writer.vu57((this.time = model.clock.time));
        this.cRoot(model.root);
    }
    encodeClockTable(clockTableOffset) {
        this.writer.view.setUint32(clockTableOffset, this.writer.x - clockTableOffset - 4);
        const clockEncoder = this.clockEncoder;
        const table = clockEncoder.table;
        const length = table.size;
        this.writer.vu57(length);
        table.forEach(this.cTableEntry);
    }
    cTableEntry = (entry) => {
        const clock = entry.clock;
        this.writer.u53vu39(clock.sid, clock.time);
    };
    tsLogical = (ts) => {
        const relativeId = this.clockEncoder.append(ts);
        this.writer.id(relativeId.sessionIndex, relativeId.timeDiff);
    };
    tsServer = (ts) => {
        this.writer.vu57(ts.time);
    };
    ts = this.tsLogical;
    cRoot(root) {
        const val = root.val;
        if (val.sid === 0)
            this.writer.u8(0);
        else
            this.cNode(root.node());
    }
    cNode(node) {
        if (node instanceof Const)
            this.cConst(node);
        else if (node instanceof ValueLww)
            this.cVal(node);
        else if (node instanceof StringRga)
            this.cStr(node);
        else if (node instanceof ObjectLww)
            this.cObj(node);
        else if (node instanceof ArrayRga)
            this.cArr(node);
        else if (node instanceof BinaryRga)
            this.cBin(node);
    }
    cObj(obj) {
        this.ts(obj.id);
        const latest = obj.latest;
        this.encodeObjectHeader(latest.size);
        obj.latest.forEach(this.cKey);
    }
    cKey = (val, key) => {
        this.encodeString(key);
        this.cNode(this.doc.index.get(val));
    };
    cArr(obj) {
        const ts = this.ts;
        const writer = this.writer;
        ts(obj.id);
        this.encodeArrayHeader(obj.size());
        const index = this.doc.index;
        for (let chunk = obj.first(); chunk; chunk = obj.next(chunk)) {
            const span = chunk.span;
            const deleted = chunk.del;
            writer.b1vu28(deleted, span);
            ts(chunk.id);
            if (deleted)
                continue;
            const nodes = chunk.data;
            for (let i = 0; i < span; i++)
                this.cNode(index.get(nodes[i]));
        }
    }
    cStr(obj) {
        const ts = this.ts;
        const writer = this.writer;
        ts(obj.id);
        const length = obj.size();
        this.encodeStringHeader(length);
        for (let chunk = obj.first(); chunk; chunk = obj.next(chunk)) {
            ts(chunk.id);
            if (chunk.del) {
                writer.u8(0);
                writer.vu39(chunk.span);
            }
            else
                this.encodeString(chunk.data);
        }
    }
    cBin(obj) {
        const ts = this.ts;
        const writer = this.writer;
        ts(obj.id);
        const length = obj.size();
        this.encodeBinaryHeader(length);
        for (let chunk = obj.first(); chunk; chunk = obj.next(chunk)) {
            const length = chunk.span;
            const deleted = chunk.del;
            writer.b1vu28(chunk.del, length);
            ts(chunk.id);
            if (deleted)
                continue;
            writer.buf(chunk.data, length);
        }
    }
    cVal(obj) {
        this.ts(obj.id);
        this.writer.u8(0xd6);
        this.cNode(obj.node());
    }
    cConst(obj) {
        this.ts(obj.id);
        this.writer.u8(0xd4);
        this.encodeAny(obj.val);
    }
}
