import { CrdtDecoder } from '../../../../json-crdt-patch/util/binary/CrdtDecoder';
import { MsgPackDecoderFast } from '../../../../json-pack/msgpack';
export class ViewDecoder extends MsgPackDecoderFast {
    time = -1;
    constructor() {
        super(new CrdtDecoder());
    }
    decode(data) {
        this.time = -1;
        this.reader.reset(data);
        const isServerTime = this.reader.u8() === 0;
        if (isServerTime) {
            this.time = this.reader.vu57();
        }
        else {
            this.reader.x += 4;
        }
        return this.cRoot();
    }
    ts() {
        if (this.time < 0)
            this.reader.idSkip();
        else
            this.reader.vu57Skip();
    }
    cRoot() {
        const peek = this.reader.uint8[this.reader.x];
        return !peek ? undefined : this.cNode();
    }
    cNode() {
        this.ts();
        const byte = this.reader.u8();
        if (byte <= 0b10001111)
            return this.cObj(byte & 0b1111);
        else if (byte <= 0b10011111)
            return this.cArr(byte & 0b1111);
        else if (byte <= 0b10111111)
            return this.cStr(byte & 0b11111);
        else {
            switch (byte) {
                case 0xc4:
                    return this.cBin(this.reader.u8());
                case 0xc5:
                    return this.cBin(this.reader.u16());
                case 0xc6:
                    return this.cBin(this.reader.u32());
                case 0xd4:
                    return this.cConst();
                case 0xd6:
                    return this.cVal();
                case 0xde:
                    return this.cObj(this.reader.u16());
                case 0xdf:
                    return this.cObj(this.reader.u32());
                case 0xdc:
                    return this.cArr(this.reader.u16());
                case 0xdd:
                    return this.cArr(this.reader.u32());
                case 0xd9:
                    return this.cStr(this.reader.u8());
                case 0xda:
                    return this.cStr(this.reader.u16());
                case 0xdb:
                    return this.cStr(this.reader.u32());
            }
        }
        return undefined;
    }
    cObj(length) {
        const obj = {};
        for (let i = 0; i < length; i++) {
            const key = this.val();
            const value = this.cNode();
            if (value !== undefined)
                obj[key] = value;
        }
        return obj;
    }
    cArr(length) {
        const arr = [];
        for (let i = 0; i < length; i++) {
            const values = this.cArrChunk();
            if (values && values.length)
                arr.push(...values);
        }
        return arr;
    }
    cArrChunk() {
        const [deleted, length] = this.reader.b1vu56();
        this.ts();
        if (deleted) {
            return undefined;
        }
        else {
            const values = [];
            for (let i = 0; i < length; i++)
                values.push(this.cNode());
            return values;
        }
    }
    cStr(length) {
        let str = '';
        for (let i = 0; i < length; i++) {
            this.ts();
            const isTombstone = this.reader.uint8[this.reader.x] === 0;
            if (isTombstone) {
                this.reader.x++;
                this.reader.vu57Skip();
                continue;
            }
            const text = this.val();
            str += text;
        }
        return str;
    }
    cBin(length) {
        const buffers = [];
        let totalLength = 0;
        for (let i = 0; i < length; i++) {
            const [deleted, length] = this.reader.b1vu56();
            this.ts();
            if (deleted)
                continue;
            buffers.push(this.bin(length));
            totalLength += length;
        }
        const res = new Uint8Array(totalLength);
        let offset = 0;
        for (let i = 0; i < buffers.length; i++) {
            const byteLength = buffers[i].byteLength;
            res.set(buffers[i], offset);
            offset += byteLength;
        }
        return res;
    }
    cVal() {
        return this.cNode();
    }
    cConst() {
        return this.val();
    }
}
