import { ArrayApi } from './ArrayApi';
import { ArrayRga } from '../../types/rga-array/ArrayRga';
import { BinaryApi } from './BinaryApi';
import { BinaryRga } from '../../types/rga-binary/BinaryRga';
import { ObjectApi } from './ObjectApi';
import { ObjectLww } from '../../types/lww-object/ObjectLww';
import { toPath } from '../../../json-pointer';
import { StringApi } from './StringApi';
import { StringRga } from '../../types/rga-string/StringRga';
import { ValueApi } from './ValueApi';
import { ValueLww } from '../../types/lww-value/ValueLww';
import { Const } from '../../types/const/Const';
import { ConstApi } from './ConstApi';
export class Finder {
    node;
    api;
    constructor(node, api) {
        this.node = node;
        this.api = api;
    }
    find(path) {
        const steps = toPath(path);
        let node = this.node;
        const length = steps.length;
        if (!length)
            return node;
        let i = 0;
        while (i < length && node) {
            const step = steps[i++];
            node = node.container();
            if (!node)
                throw new Error('NOT_CONTAINER');
            if (node instanceof ObjectLww) {
                const nextNode = node.get(String(step));
                if (!nextNode)
                    throw new Error('NOT_FOUND');
                node = nextNode;
            }
            else if (node instanceof ArrayRga) {
                const nextNode = node.getNode(Number(step));
                if (!nextNode)
                    throw new Error('NOT_FOUND');
                node = nextNode;
            }
        }
        return node;
    }
    val(path) {
        const node = this.find(path);
        if (node instanceof ValueLww)
            return new ValueApi(this.api, node);
        throw new Error('NOT_VAL');
    }
    str(path) {
        const node = this.find(path);
        if (node instanceof StringRga)
            return new StringApi(this.api, node);
        throw new Error('NOT_STR');
    }
    bin(path) {
        const node = this.find(path);
        if (node instanceof BinaryRga)
            return new BinaryApi(this.api, node);
        throw new Error('NOT_BIN');
    }
    arr(path) {
        const node = this.find(path);
        if (node instanceof ArrayRga)
            return new ArrayApi(this.api, node);
        throw new Error('NOT_ARR');
    }
    obj(path) {
        const obj = this.find(path);
        if (obj instanceof ObjectLww)
            return new ObjectApi(this.api, obj);
        throw new Error('NOT_OBJ');
    }
    const(path) {
        const obj = this.find(path);
        if (obj instanceof Const)
            return new ConstApi(obj);
        throw new Error('NOT_CONST');
    }
}
