import { AbstractRga } from '../rga/AbstractRga';
import { tick } from '../../../json-crdt-patch/clock';
const Empty = [];
export class ArrayChunk {
    id;
    span;
    del;
    data;
    len;
    p;
    l;
    r;
    s;
    constructor(id, span, data) {
        this.id = id;
        this.span = span;
        this.len = data ? span : 0;
        this.del = !data;
        this.p = undefined;
        this.l = undefined;
        this.r = undefined;
        this.s = undefined;
        this.data = data;
    }
    merge(data) {
        this.data.push(...data);
        this.span = this.data.length;
    }
    split(ticks) {
        const span = this.span;
        this.span = ticks;
        if (!this.del) {
            const data = this.data;
            const rightData = data.splice(ticks);
            const chunk = new ArrayChunk(tick(this.id, ticks), span - ticks, rightData);
            return chunk;
        }
        return new ArrayChunk(tick(this.id, ticks), span - ticks, undefined);
    }
    delete() {
        this.del = true;
        this.data = undefined;
    }
    clone() {
        return new ArrayChunk(this.id, this.span, this.data ? [...this.data] : undefined);
    }
}
export class ArrayRga extends AbstractRga {
    doc;
    constructor(doc, id) {
        super(id);
        this.doc = doc;
    }
    create() {
        return new ArrayRga(this.doc, this.id);
    }
    createChunk(id, data) {
        return new ArrayChunk(id, data ? data.length : 0, data);
    }
    onViewChange() {
        this._view = Empty;
    }
    _view = Empty;
    view() {
        const arr = [];
        let chunk = this.first();
        const index = this.doc.index;
        while (chunk) {
            if (!chunk.del)
                for (const node of chunk.data)
                    arr.push(index.get(node).view());
            chunk = this.next(chunk);
        }
        const _view = this._view;
        if (arr.length !== _view.length)
            return (this._view = arr);
        for (let i = 0; i < arr.length; i++)
            if (arr[i] !== _view[i])
                return (this._view = arr);
        return _view;
    }
    children(callback) {
        const index = this.doc.index;
        for (let chunk = this.first(); chunk; chunk = this.next(chunk))
            if (!chunk.del)
                for (const node of chunk.data)
                    callback(index.get(node));
    }
    formatChunk(chunk, tab, side) {
        let str = super.formatChunk(chunk, tab, side);
        if (chunk.del)
            return str;
        let i = this.pos(chunk);
        const last = chunk.data.length - 1;
        for (let j = 0; j <= last; j++) {
            const node = this.doc.index.get(chunk.data[j]);
            const isLast = j === last;
            const index = String(i++);
            const chunkTab = tab + `   ${isLast ? ' ' : '│'}      ` + ' '.repeat(index.length);
            str += `\n${tab}   ${isLast ? '└─' : '├─'} [${index}]: ${node.toString(chunkTab)}`;
        }
        return str;
    }
    get(position) {
        const pair = this.findChunk(position);
        if (!pair)
            return undefined;
        return pair[0].data[pair[1]];
    }
    getNode(position) {
        const id = this.get(position);
        if (!id)
            return undefined;
        return this.doc.index.get(id);
    }
    getById(id) {
        const chunk = this.findById(id);
        if (!chunk || chunk.del)
            return undefined;
        const offset = id.time - chunk.id.time;
        return chunk.data[offset];
    }
    container() {
        return this;
    }
}
