import { Writer } from '../../util/buffers/Writer';
const isSafeInteger = Number.isSafeInteger;
export class IonEncoderFast {
    writer;
    constructor(writer = new Writer()) {
        this.writer = writer;
    }
    encode(value) {
        this.writer.reset();
        this.writeIvm();
        this.writeAny(value);
        return this.writer.flush();
    }
    writeAny(value) {
        switch (value) {
            case null:
                return this.writer.u8(0 + 15);
        }
        switch (typeof value) {
            case 'number':
                return this.encodeNumber(value);
            case 'boolean':
                return this.writeBool(value);
        }
    }
    writeIvm() {
        this.writer.u32(0xe00100ea);
    }
    writeBool(bool) {
        this.writer.u8(16 + (bool ? 1 : 0));
    }
    encodeNumber(num) {
        if (isSafeInteger(num))
            this.encodeInteger(num);
        else
            this.writeFloat(num);
    }
    encodeInteger(int) {
        if (int >= 0)
            this.encodeUint(int);
        else
            this.encodeNint(int);
    }
    encodeUint(uint) {
        if (!uint)
            this.writer.u8(32);
        else if (uint <= 0xff)
            this.writer.u16(((32 + 1) << 8) + uint);
        else if (uint <= 0xffff)
            this.writer.u8u16(32 + 2, uint);
        else if (uint <= 0xffffff)
            this.writer.u32(((32 + 3) << 24) + uint);
        else if (uint <= 0xffffffff)
            this.writer.u8u32(32 + 4, uint);
        else {
            let lo = uint | 0;
            if (lo < 0)
                lo += 4294967296;
            let hi = uint - lo;
            hi /= 4294967296;
            if (uint <= 0xffffffffff) {
                this.writer.u16(((32 + 5) << 8) + hi);
                this.writer.u32(lo);
            }
            else if (uint <= 0xffffffffffff) {
                this.writer.u8u16(32 + 6, hi);
                this.writer.u32(lo);
            }
            else {
                this.writer.u16(((32 + 7) << 8) + (hi >> 16));
                this.writer.u16(hi & 0xffff);
                this.writer.u32(lo);
            }
        }
    }
    encodeNint(int) {
        const uint = -int;
        if (uint <= 0xff)
            this.writer.u16(((48 + 1) << 8) + uint);
        else if (uint <= 0xffff)
            this.writer.u8u16(48 + 2, uint);
        else if (uint <= 0xffffff)
            this.writer.u32(((48 + 3) << 24) + uint);
        else if (uint <= 0xffffffff)
            this.writer.u8u32(48 + 4, uint);
        else {
            let lo = uint | 0;
            if (lo < 0)
                lo += 4294967296;
            let hi = uint - lo;
            hi /= 4294967296;
            if (uint <= 0xffffffffff) {
                this.writer.u16(((48 + 5) << 8) + hi);
                this.writer.u32(lo);
            }
            else if (uint <= 0xffffffffffff) {
                this.writer.u8u16(48 + 6, hi);
                this.writer.u32(lo);
            }
            else {
                this.writer.u16(((48 + 7) << 8) + (hi >> 16));
                this.writer.u16(hi & 0xffff);
                this.writer.u32(lo);
            }
        }
    }
    writeFloat(float) {
        this.writer.u8f64(64 + 8, float);
    }
}
