"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var ClockTable_1 = require("../../../../json-crdt-patch/codec/clock/ClockTable");
var CrdtEncoder_1 = require("../../../../json-crdt-patch/util/binary/CrdtEncoder");
var msgpack_1 = require("../../../../json-pack/msgpack");
var Const_1 = require("../../../types/const/Const");
var ObjectLww_1 = require("../../../types/lww-object/ObjectLww");
var ValueLww_1 = require("../../../types/lww-value/ValueLww");
var ArrayRga_1 = require("../../../types/rga-array/ArrayRga");
var BinaryRga_1 = require("../../../types/rga-binary/BinaryRga");
var StringRga_1 = require("../../../types/rga-string/StringRga");
var EMPTY = new Uint8Array(0);
var Encoder = (function () {
    function Encoder() {
        var _this = this;
        this.enc = new msgpack_1.MsgPackEncoder(new CrdtEncoder_1.CrdtWriter());
        this.onNode = function (map, sid) {
            var model = _this.model;
            var sidIndex = _this.clockTable.getBySid(sid).index;
            var sidFieldPart = sidIndex.toString(36) + '_';
            map.forEach(function (node, time) {
                var field = (sidFieldPart + time.toString(36));
                model[field] = _this.encodeNode(node);
            });
        };
        this.onObjectKey = function (value, key) {
            _this.enc.encodeString(key);
            _this.ts(value);
        };
    }
    Encoder.prototype.encode = function (doc, clockTable) {
        if (clockTable === void 0) { clockTable = ClockTable_1.ClockTable.from(doc.clock); }
        this.clockTable = clockTable;
        var writer = this.enc.writer;
        writer.reset();
        clockTable.write(writer);
        var encodedClock = writer.flush();
        var rootValueId = doc.root.val;
        var model = (this.model = {
            c: encodedClock,
        });
        if (rootValueId.sid !== 0) {
            writer.reset();
            this.ts(rootValueId);
            model.r = writer.flush();
        }
        doc.index.entries.forEach(this.onNode);
        return model;
    };
    Encoder.prototype.encodeNode = function (node) {
        if (node instanceof ValueLww_1.ValueLww)
            return this.encodeVal(node);
        else if (node instanceof Const_1.Const)
            return this.encodeConst(node);
        else if (node instanceof StringRga_1.StringRga)
            return this.encodeStr(node);
        else if (node instanceof ObjectLww_1.ObjectLww)
            return this.encodeObj(node);
        else if (node instanceof ArrayRga_1.ArrayRga)
            return this.encodeArr(node);
        else if (node instanceof BinaryRga_1.BinaryRga)
            return this.encodeBin(node);
        else
            return EMPTY;
    };
    Encoder.prototype.ts = function (id) {
        var index = this.clockTable.getBySid(id.sid).index;
        this.enc.writer.id(index, id.time);
    };
    Encoder.prototype.encodeVal = function (node) {
        var writer = this.enc.writer;
        var child = node.node();
        writer.reset();
        writer.u8(0xd6);
        this.ts(child.id);
        return writer.flush();
    };
    Encoder.prototype.encodeConst = function (node) {
        var encoder = this.enc;
        var writer = encoder.writer;
        writer.reset();
        writer.u8(0xd4);
        encoder.encodeAny(node.val);
        return writer.flush();
    };
    Encoder.prototype.encodeStr = function (node) {
        var encoder = this.enc;
        var writer = encoder.writer;
        writer.reset();
        encoder.encodeStringHeader(node.size());
        for (var chunk = node.first(); chunk; chunk = node.next(chunk)) {
            this.ts(chunk.id);
            if (chunk.del)
                encoder.encodeUnsignedInteger(chunk.span);
            else
                encoder.encodeString(chunk.data);
        }
        return writer.flush();
    };
    Encoder.prototype.encodeBin = function (node) {
        var encoder = this.enc;
        var writer = encoder.writer;
        writer.reset();
        encoder.encodeBinaryHeader(node.size());
        for (var chunk = node.first(); chunk; chunk = node.next(chunk)) {
            this.ts(chunk.id);
            var length_1 = chunk.span;
            if (chunk.del) {
                writer.b1vu56(true, length_1);
            }
            else {
                writer.b1vu56(false, length_1);
                writer.buf(chunk.data, length_1);
            }
        }
        return writer.flush();
    };
    Encoder.prototype.encodeObj = function (node) {
        var encoder = this.enc;
        var writer = encoder.writer;
        writer.reset();
        encoder.encodeObjectHeader(node.latest.size);
        node.latest.forEach(this.onObjectKey);
        return writer.flush();
    };
    Encoder.prototype.encodeArr = function (node) {
        var encoder = this.enc;
        var writer = encoder.writer;
        writer.reset();
        encoder.encodeArrayHeader(node.size());
        for (var chunk = node.first(); chunk; chunk = node.next(chunk)) {
            var length_2 = chunk.span;
            var deleted = chunk.del;
            this.ts(chunk.id);
            writer.b1vu28(deleted, length_2);
            if (deleted)
                continue;
            var data = chunk.data;
            for (var i = 0; i < length_2; i++)
                this.ts(data[i]);
        }
        return writer.flush();
    };
    return Encoder;
}());
exports.Encoder = Encoder;
