"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Decoder = void 0;
var tslib_1 = require("tslib");
var ArrayRga_1 = require("../../../types/rga-array/ArrayRga");
var BinaryRga_1 = require("../../../types/rga-binary/BinaryRga");
var ClockDecoder_1 = require("../../../../json-crdt-patch/codec/clock/ClockDecoder");
var Const_1 = require("../../../types/const/Const");
var CrdtDecoder_1 = require("../../../../json-crdt-patch/util/binary/CrdtDecoder");
var clock_1 = require("../../../../json-crdt-patch/clock");
var model_1 = require("../../../model");
var msgpack_1 = require("../../../../json-pack/msgpack");
var ObjectLww_1 = require("../../../types/lww-object/ObjectLww");
var RootLww_1 = require("../../../types/lww-root/RootLww");
var StringRga_1 = require("../../../types/rga-string/StringRga");
var ValueLww_1 = require("../../../types/lww-value/ValueLww");
var Decoder = (function (_super) {
    tslib_1.__extends(Decoder, _super);
    function Decoder() {
        var _this = _super.call(this, new CrdtDecoder_1.CrdtDecoder()) || this;
        _this.time = -1;
        _this.cArrChunk = function () {
            var _a = tslib_1.__read(_this.reader.b1vu28(), 2), deleted = _a[0], length = _a[1];
            var id = _this.ts();
            if (deleted)
                return new ArrayRga_1.ArrayChunk(id, length, undefined);
            var ids = [];
            for (var i = 0; i < length; i++)
                ids.push(_this.cNode().id);
            return new ArrayRga_1.ArrayChunk(id, length, ids);
        };
        _this.cStrChunk = function () {
            var id = _this.ts();
            var isTombstone = _this.reader.uint8[_this.reader.x] === 0;
            if (isTombstone) {
                _this.reader.x++;
                var length_1 = _this.reader.vu39();
                return new StringRga_1.StringChunk(id, length_1, '');
            }
            var text = _this.val();
            return new StringRga_1.StringChunk(id, text.length, text);
        };
        _this.cBinChunk = function () {
            var _a = tslib_1.__read(_this.reader.b1vu28(), 2), deleted = _a[0], length = _a[1];
            var id = _this.ts();
            if (deleted)
                return new BinaryRga_1.BinaryChunk(id, length, undefined);
            else
                return new BinaryRga_1.BinaryChunk(id, length, _this.bin(length));
        };
        return _this;
    }
    Decoder.prototype.decode = function (data) {
        delete this.clockDecoder;
        this.time = -1;
        this.reader.reset(data);
        var isServerTime = this.reader.u8() === 0;
        if (isServerTime) {
            var time = (this.time = this.reader.vu57());
            this.doc = model_1.Model.withServerClock(time);
        }
        else {
            this.decodeClockTable();
            var clock = this.clockDecoder.clock;
            this.doc = model_1.Model.withLogicalClock(clock);
        }
        this.doc.root = new RootLww_1.RootLww(this.doc, this.cRoot().id);
        delete this.clockDecoder;
        return this.doc;
    };
    Decoder.prototype.decodeClockTable = function () {
        var clockTableOffset = this.reader.u32();
        var offset = this.reader.x;
        this.reader.x += clockTableOffset;
        var length = this.reader.vu57();
        var _a = tslib_1.__read(this.reader.u53vu39(), 2), sessionId = _a[0], time = _a[1];
        this.clockDecoder = new ClockDecoder_1.ClockDecoder(sessionId, time);
        for (var i = 1; i < length; i++) {
            var _b = tslib_1.__read(this.reader.u53vu39(), 2), sid = _b[0], time_1 = _b[1];
            this.clockDecoder.pushTuple(sid, time_1);
        }
        this.reader.x = offset;
    };
    Decoder.prototype.ts = function () {
        var decoderTime = this.time;
        var isLogical = decoderTime < 0;
        if (isLogical) {
            var _a = tslib_1.__read(this.reader.id(), 2), sessionIndex = _a[0], timeDiff = _a[1];
            return this.clockDecoder.decodeId(sessionIndex, timeDiff);
        }
        else {
            return new clock_1.Timestamp(1, this.reader.vu57());
        }
    };
    Decoder.prototype.cRoot = function () {
        var peek = this.reader.uint8[this.reader.x];
        return !peek ? model_1.UNDEFINED : this.cNode();
    };
    Decoder.prototype.cNode = function () {
        var id = this.ts();
        var byte = this.reader.u8();
        if (byte <= 143)
            return this.cObj(id, byte & 15);
        else if (byte <= 159)
            return this.cArr(id, byte & 15);
        else if (byte <= 191)
            return this.cStr(id, byte & 31);
        else {
            switch (byte) {
                case 0xc4:
                    return this.cBin(id, this.reader.u8());
                case 0xc5:
                    return this.cBin(id, this.reader.u16());
                case 0xc6:
                    return this.cBin(id, this.reader.u32());
                case 0xd4:
                    return this.cConst(id);
                case 0xd6:
                    return this.cVal(id);
                case 0xde:
                    return this.cObj(id, this.reader.u16());
                case 0xdf:
                    return this.cObj(id, this.reader.u32());
                case 0xdc:
                    return this.cArr(id, this.reader.u16());
                case 0xdd:
                    return this.cArr(id, this.reader.u32());
                case 0xd9:
                    return this.cStr(id, this.reader.u8());
                case 0xda:
                    return this.cStr(id, this.reader.u16());
                case 0xdb:
                    return this.cStr(id, this.reader.u32());
            }
        }
        throw new Error('UNKNOWN_NODE');
    };
    Decoder.prototype.cObj = function (id, length) {
        var obj = new ObjectLww_1.ObjectLww(this.doc, id);
        for (var i = 0; i < length; i++)
            this.cObjChunk(obj);
        this.doc.index.set(obj);
        return obj;
    };
    Decoder.prototype.cObjChunk = function (obj) {
        var key = this.val();
        obj.put(key, this.cNode().id);
    };
    Decoder.prototype.cArr = function (id, length) {
        var obj = new ArrayRga_1.ArrayRga(this.doc, id);
        obj.ingest(length, this.cArrChunk);
        this.doc.index.set(obj);
        return obj;
    };
    Decoder.prototype.cStr = function (id, length) {
        var node = new StringRga_1.StringRga(id);
        if (length)
            node.ingest(length, this.cStrChunk);
        this.doc.index.set(node);
        return node;
    };
    Decoder.prototype.cBin = function (id, length) {
        var node = new BinaryRga_1.BinaryRga(id);
        if (length)
            node.ingest(length, this.cBinChunk);
        this.doc.index.set(node);
        return node;
    };
    Decoder.prototype.cConst = function (id) {
        var obj = new Const_1.Const(id, this.val());
        this.doc.index.set(obj);
        return obj;
    };
    Decoder.prototype.cVal = function (id) {
        var val = this.cNode();
        var obj = new ValueLww_1.ValueLww(this.doc, id, val.id);
        this.doc.index.set(obj);
        return obj;
    };
    return Decoder;
}(msgpack_1.MsgPackDecoderFast));
exports.Decoder = Decoder;
