"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var tslib_1 = require("tslib");
var ArrayRga_1 = require("../../../types/rga-array/ArrayRga");
var BinaryRga_1 = require("../../../types/rga-binary/BinaryRga");
var ClockEncoder_1 = require("../../../../json-crdt-patch/codec/clock/ClockEncoder");
var Const_1 = require("../../../types/const/Const");
var CrdtEncoder_1 = require("../../../../json-crdt-patch/util/binary/CrdtEncoder");
var msgpack_1 = require("../../../../json-pack/msgpack");
var ObjectLww_1 = require("../../../types/lww-object/ObjectLww");
var StringRga_1 = require("../../../types/rga-string/StringRga");
var ValueLww_1 = require("../../../types/lww-value/ValueLww");
var Encoder = (function (_super) {
    tslib_1.__extends(Encoder, _super);
    function Encoder() {
        var _this = _super.call(this, new CrdtEncoder_1.CrdtWriter()) || this;
        _this.clockEncoder = new ClockEncoder_1.ClockEncoder();
        _this.time = 0;
        _this.cTableEntry = function (entry) {
            var clock = entry.clock;
            _this.writer.u53vu39(clock.sid, clock.time);
        };
        _this.tsLogical = function (ts) {
            var relativeId = _this.clockEncoder.append(ts);
            _this.writer.id(relativeId.sessionIndex, relativeId.timeDiff);
        };
        _this.tsServer = function (ts) {
            _this.writer.vu57(ts.time);
        };
        _this.ts = _this.tsLogical;
        _this.cKey = function (val, key) {
            _this.encodeString(key);
            _this.cNode(_this.doc.index.get(val));
        };
        return _this;
    }
    Encoder.prototype.encode = function (doc) {
        this.doc = doc;
        this.writer.reset();
        if (doc.clock.sid === 1)
            this.encodeServer(doc);
        else
            this.encodeLogical(doc);
        return this.writer.flush();
    };
    Encoder.prototype.encodeLogical = function (model) {
        var writer = this.writer;
        this.ts = this.tsLogical;
        var clockTableOffset = 0;
        writer.u8(1);
        this.clockEncoder.reset(model.clock);
        clockTableOffset = writer.x;
        writer.ensureCapacity(4);
        writer.x += 4;
        this.cRoot(model.root);
        this.encodeClockTable(clockTableOffset);
    };
    Encoder.prototype.encodeServer = function (model) {
        this.ts = this.tsServer;
        this.writer.u8(0);
        this.writer.vu57((this.time = model.clock.time));
        this.cRoot(model.root);
    };
    Encoder.prototype.encodeClockTable = function (clockTableOffset) {
        this.writer.view.setUint32(clockTableOffset, this.writer.x - clockTableOffset - 4);
        var clockEncoder = this.clockEncoder;
        var table = clockEncoder.table;
        var length = table.size;
        this.writer.vu57(length);
        table.forEach(this.cTableEntry);
    };
    Encoder.prototype.cRoot = function (root) {
        var val = root.val;
        if (val.sid === 0)
            this.writer.u8(0);
        else
            this.cNode(root.node());
    };
    Encoder.prototype.cNode = function (node) {
        if (node instanceof Const_1.Const)
            this.cConst(node);
        else if (node instanceof ValueLww_1.ValueLww)
            this.cVal(node);
        else if (node instanceof StringRga_1.StringRga)
            this.cStr(node);
        else if (node instanceof ObjectLww_1.ObjectLww)
            this.cObj(node);
        else if (node instanceof ArrayRga_1.ArrayRga)
            this.cArr(node);
        else if (node instanceof BinaryRga_1.BinaryRga)
            this.cBin(node);
    };
    Encoder.prototype.cObj = function (obj) {
        this.ts(obj.id);
        var latest = obj.latest;
        this.encodeObjectHeader(latest.size);
        obj.latest.forEach(this.cKey);
    };
    Encoder.prototype.cArr = function (obj) {
        var ts = this.ts;
        var writer = this.writer;
        ts(obj.id);
        this.encodeArrayHeader(obj.size());
        var index = this.doc.index;
        for (var chunk = obj.first(); chunk; chunk = obj.next(chunk)) {
            var span = chunk.span;
            var deleted = chunk.del;
            writer.b1vu28(deleted, span);
            ts(chunk.id);
            if (deleted)
                continue;
            var nodes = chunk.data;
            for (var i = 0; i < span; i++)
                this.cNode(index.get(nodes[i]));
        }
    };
    Encoder.prototype.cStr = function (obj) {
        var ts = this.ts;
        var writer = this.writer;
        ts(obj.id);
        var length = obj.size();
        this.encodeStringHeader(length);
        for (var chunk = obj.first(); chunk; chunk = obj.next(chunk)) {
            ts(chunk.id);
            if (chunk.del) {
                writer.u8(0);
                writer.vu39(chunk.span);
            }
            else
                this.encodeString(chunk.data);
        }
    };
    Encoder.prototype.cBin = function (obj) {
        var ts = this.ts;
        var writer = this.writer;
        ts(obj.id);
        var length = obj.size();
        this.encodeBinaryHeader(length);
        for (var chunk = obj.first(); chunk; chunk = obj.next(chunk)) {
            var length_1 = chunk.span;
            var deleted = chunk.del;
            writer.b1vu28(chunk.del, length_1);
            ts(chunk.id);
            if (deleted)
                continue;
            writer.buf(chunk.data, length_1);
        }
    };
    Encoder.prototype.cVal = function (obj) {
        this.ts(obj.id);
        this.writer.u8(0xd6);
        this.cNode(obj.node());
    };
    Encoder.prototype.cConst = function (obj) {
        this.ts(obj.id);
        this.writer.u8(0xd4);
        this.encodeAny(obj.val);
    };
    return Encoder;
}(msgpack_1.MsgPackEncoder));
exports.Encoder = Encoder;
