"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViewDecoder = void 0;
var tslib_1 = require("tslib");
var CrdtDecoder_1 = require("../../../../json-crdt-patch/util/binary/CrdtDecoder");
var msgpack_1 = require("../../../../json-pack/msgpack");
var ViewDecoder = (function (_super) {
    tslib_1.__extends(ViewDecoder, _super);
    function ViewDecoder() {
        var _this = _super.call(this, new CrdtDecoder_1.CrdtDecoder()) || this;
        _this.time = -1;
        return _this;
    }
    ViewDecoder.prototype.decode = function (data) {
        this.time = -1;
        this.reader.reset(data);
        var isServerTime = this.reader.u8() === 0;
        if (isServerTime) {
            this.time = this.reader.vu57();
        }
        else {
            this.reader.x += 4;
        }
        return this.cRoot();
    };
    ViewDecoder.prototype.ts = function () {
        if (this.time < 0)
            this.reader.idSkip();
        else
            this.reader.vu57Skip();
    };
    ViewDecoder.prototype.cRoot = function () {
        var peek = this.reader.uint8[this.reader.x];
        return !peek ? undefined : this.cNode();
    };
    ViewDecoder.prototype.cNode = function () {
        this.ts();
        var byte = this.reader.u8();
        if (byte <= 143)
            return this.cObj(byte & 15);
        else if (byte <= 159)
            return this.cArr(byte & 15);
        else if (byte <= 191)
            return this.cStr(byte & 31);
        else {
            switch (byte) {
                case 0xc4:
                    return this.cBin(this.reader.u8());
                case 0xc5:
                    return this.cBin(this.reader.u16());
                case 0xc6:
                    return this.cBin(this.reader.u32());
                case 0xd4:
                    return this.cConst();
                case 0xd6:
                    return this.cVal();
                case 0xde:
                    return this.cObj(this.reader.u16());
                case 0xdf:
                    return this.cObj(this.reader.u32());
                case 0xdc:
                    return this.cArr(this.reader.u16());
                case 0xdd:
                    return this.cArr(this.reader.u32());
                case 0xd9:
                    return this.cStr(this.reader.u8());
                case 0xda:
                    return this.cStr(this.reader.u16());
                case 0xdb:
                    return this.cStr(this.reader.u32());
            }
        }
        return undefined;
    };
    ViewDecoder.prototype.cObj = function (length) {
        var obj = {};
        for (var i = 0; i < length; i++) {
            var key = this.val();
            var value = this.cNode();
            if (value !== undefined)
                obj[key] = value;
        }
        return obj;
    };
    ViewDecoder.prototype.cArr = function (length) {
        var arr = [];
        for (var i = 0; i < length; i++) {
            var values = this.cArrChunk();
            if (values && values.length)
                arr.push.apply(arr, tslib_1.__spreadArray([], tslib_1.__read(values), false));
        }
        return arr;
    };
    ViewDecoder.prototype.cArrChunk = function () {
        var _a = tslib_1.__read(this.reader.b1vu56(), 2), deleted = _a[0], length = _a[1];
        this.ts();
        if (deleted) {
            return undefined;
        }
        else {
            var values = [];
            for (var i = 0; i < length; i++)
                values.push(this.cNode());
            return values;
        }
    };
    ViewDecoder.prototype.cStr = function (length) {
        var str = '';
        for (var i = 0; i < length; i++) {
            this.ts();
            var isTombstone = this.reader.uint8[this.reader.x] === 0;
            if (isTombstone) {
                this.reader.x++;
                this.reader.vu57Skip();
                continue;
            }
            var text = this.val();
            str += text;
        }
        return str;
    };
    ViewDecoder.prototype.cBin = function (length) {
        var buffers = [];
        var totalLength = 0;
        for (var i = 0; i < length; i++) {
            var _a = tslib_1.__read(this.reader.b1vu56(), 2), deleted = _a[0], length_1 = _a[1];
            this.ts();
            if (deleted)
                continue;
            buffers.push(this.bin(length_1));
            totalLength += length_1;
        }
        var res = new Uint8Array(totalLength);
        var offset = 0;
        for (var i = 0; i < buffers.length; i++) {
            var byteLength = buffers[i].byteLength;
            res.set(buffers[i], offset);
            offset += byteLength;
        }
        return res;
    };
    ViewDecoder.prototype.cVal = function () {
        return this.cNode();
    };
    ViewDecoder.prototype.cConst = function () {
        return this.val();
    };
    return ViewDecoder;
}(msgpack_1.MsgPackDecoderFast));
exports.ViewDecoder = ViewDecoder;
