"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Decoder = void 0;
var tslib_1 = require("tslib");
var ArrayRga_1 = require("../../../types/rga-array/ArrayRga");
var BinaryRga_1 = require("../../../types/rga-binary/BinaryRga");
var ClockDecoder_1 = require("../../../../json-crdt-patch/codec/clock/ClockDecoder");
var Const_1 = require("../../../types/const/Const");
var RootLww_1 = require("../../../types/lww-root/RootLww");
var clock_1 = require("../../../../json-crdt-patch/clock");
var Model_1 = require("../../../model/Model");
var ObjectLww_1 = require("../../../types/lww-object/ObjectLww");
var constants_1 = require("../../../../json-crdt-patch/constants");
var StringRga_1 = require("../../../types/rga-string/StringRga");
var ValueLww_1 = require("../../../types/lww-value/ValueLww");
var Decoder = (function () {
    function Decoder() {
    }
    Decoder.prototype.decode = function (data) {
        var x = data[0];
        var isServerTime = typeof x === 'number';
        if (isServerTime) {
            this.time = x;
        }
        else {
            this.clockDecoder = ClockDecoder_1.ClockDecoder.fromArr(x);
        }
        var doc = isServerTime ? Model_1.Model.withServerClock(x) : Model_1.Model.withLogicalClock(this.clockDecoder.clock);
        var val = data[1] ? this.decodeNode(doc, data[1]) : Model_1.UNDEFINED;
        doc.root = new RootLww_1.RootLww(doc, val.id);
        return doc;
    };
    Decoder.prototype.ts = function (arr, index) {
        var x = arr[index];
        if (typeof x === 'number') {
            if (x < 0) {
                var sessionIndex = -x;
                var timeDiff = arr[index + 1];
                return [this.clockDecoder.decodeId(sessionIndex, timeDiff), index + 2];
            }
            else {
                return [new clock_1.Timestamp(1, this.time - x), index + 1];
            }
        }
        else {
            var time = x[0];
            switch (time) {
                case constants_1.ORIGIN.time:
                    return [constants_1.ORIGIN, index + 1];
                default:
                    return [new clock_1.Timestamp(0, time), index + 1];
            }
        }
    };
    Decoder.prototype.decodeNode = function (doc, data) {
        if (data instanceof Array) {
            switch (data[0]) {
                case 3:
                    return this.decodeObj(doc, data);
                case 5:
                    return this.decodeArr(doc, data);
                case 4:
                    return this.decodeStr(doc, data);
                case 2:
                    return this.decodeVal(doc, data);
                case 1:
                    return this.decodeConst(doc, data);
                case 6:
                    return this.decodeBin(doc, data);
            }
        }
        throw new Error('UNKNOWN_NODE');
    };
    Decoder.prototype.decodeObj = function (doc, data) {
        var _a = tslib_1.__read(this.ts(data, 1), 2), id = _a[0], index = _a[1];
        var obj = new ObjectLww_1.ObjectLww(doc, id);
        var length = data.length;
        for (var i = index; i < length;) {
            var key = data[i];
            var val = this.decodeNode(doc, data[++i]);
            obj.put(key, val.id);
            i++;
        }
        doc.index.set(obj);
        return obj;
    };
    Decoder.prototype.decodeArr = function (doc, data) {
        var _this = this;
        var size = data[1];
        var _a = tslib_1.__read(this.ts(data, 2), 2), id = _a[0], index = _a[1];
        var obj = new ArrayRga_1.ArrayRga(doc, id);
        var self = this;
        var i = index;
        obj.ingest(size, function () {
            var _a = tslib_1.__read(self.ts(data, i), 2), chunkId = _a[0], idx = _a[1];
            var content = data[idx];
            i = idx + 1;
            if (typeof content === 'number')
                return new ArrayRga_1.ArrayChunk(chunkId, content, undefined);
            var ids = content.map(function (c) { return _this.decodeNode(doc, c).id; });
            return new ArrayRga_1.ArrayChunk(chunkId, content.length, ids);
        });
        doc.index.set(obj);
        return obj;
    };
    Decoder.prototype.decodeStr = function (doc, data) {
        var size = data[1];
        var _a = tslib_1.__read(this.ts(data, 2), 2), id = _a[0], index = _a[1];
        var node = new StringRga_1.StringRga(id);
        var self = this;
        var i = index;
        node.ingest(size, function () {
            var _a = tslib_1.__read(self.ts(data, i), 2), chunkId = _a[0], idx = _a[1];
            var content = data[idx];
            i = idx + 1;
            if (typeof content === 'number')
                return new StringRga_1.StringChunk(chunkId, content, '');
            return new StringRga_1.StringChunk(chunkId, content.length, content);
        });
        doc.index.set(node);
        return node;
    };
    Decoder.prototype.decodeBin = function (doc, data) {
        var size = data[1];
        var _a = tslib_1.__read(this.ts(data, 2), 2), id = _a[0], index = _a[1];
        var node = new BinaryRga_1.BinaryRga(id);
        var self = this;
        var i = index;
        node.ingest(size, function () {
            var _a = tslib_1.__read(self.ts(data, i), 2), chunkId = _a[0], idx = _a[1];
            var content = data[idx];
            i = idx + 1;
            if (typeof content === 'number')
                return new BinaryRga_1.BinaryChunk(chunkId, content, undefined);
            var buf = content;
            return new BinaryRga_1.BinaryChunk(chunkId, buf.length, buf);
        });
        doc.index.set(node);
        return node;
    };
    Decoder.prototype.decodeVal = function (doc, data) {
        var _a = tslib_1.__read(this.ts(data, 1), 2), id = _a[0], index = _a[1];
        var child = this.decodeNode(doc, data[index]);
        var obj = new ValueLww_1.ValueLww(doc, id, child.id);
        doc.index.set(obj);
        return obj;
    };
    Decoder.prototype.decodeConst = function (doc, data) {
        var _a = tslib_1.__read(this.ts(data, 1), 2), id = _a[0], index = _a[1];
        var value = data[index];
        var obj = new Const_1.Const(id, value);
        doc.index.set(obj);
        return obj;
    };
    return Decoder;
}());
exports.Decoder = Decoder;
