"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var tslib_1 = require("tslib");
var ArrayRga_1 = require("../../../types/rga-array/ArrayRga");
var BinaryRga_1 = require("../../../types/rga-binary/BinaryRga");
var ClockEncoder_1 = require("../../../../json-crdt-patch/codec/clock/ClockEncoder");
var Const_1 = require("../../../types/const/Const");
var ObjectLww_1 = require("../../../types/lww-object/ObjectLww");
var StringRga_1 = require("../../../types/rga-string/StringRga");
var ValueLww_1 = require("../../../types/lww-value/ValueLww");
var Encoder = (function () {
    function Encoder() {
    }
    Encoder.prototype.encode = function (model) {
        this.model = model;
        var isServerTime = model.clock.sid === 1;
        var clock = model.clock;
        var arr = isServerTime ? [clock.time] : [null];
        if (isServerTime) {
            this.time = clock.time;
        }
        else {
            this.clock = new ClockEncoder_1.ClockEncoder();
            this.clock.reset(model.clock);
        }
        this.encodeRoot(arr, model.root);
        if (!isServerTime)
            arr[0] = this.clock.toJson();
        return arr;
    };
    Encoder.prototype.ts = function (arr, ts) {
        switch (ts.sid) {
            case 0: {
                arr.push([ts.time]);
                break;
            }
            case 1: {
                arr.push(this.time - ts.time);
                break;
            }
            default: {
                var relativeId = this.clock.append(ts);
                arr.push(-relativeId.sessionIndex, relativeId.timeDiff);
            }
        }
    };
    Encoder.prototype.encodeRoot = function (arr, root) {
        if (!root.val.time)
            arr.push(0);
        else
            this.cNode(arr, root.node());
    };
    Encoder.prototype.cNode = function (arr, node) {
        if (node instanceof ObjectLww_1.ObjectLww)
            return this.encodeObj(arr, node);
        else if (node instanceof ArrayRga_1.ArrayRga)
            return this.encodeArr(arr, node);
        else if (node instanceof StringRga_1.StringRga)
            return this.encodeStr(arr, node);
        else if (node instanceof ValueLww_1.ValueLww)
            return this.cVal(arr, node);
        else if (node instanceof Const_1.Const)
            return this.cConst(arr, node);
        else if (node instanceof BinaryRga_1.BinaryRga)
            return this.encodeBin(arr, node);
        throw new Error('UNKNOWN_NODE');
    };
    Encoder.prototype.encodeObj = function (arr, obj) {
        var _this = this;
        var res = [3];
        arr.push(res);
        this.ts(res, obj.id);
        obj.nodes(function (node, key) {
            res.push(key);
            _this.cNode(res, node);
        });
    };
    Encoder.prototype.encodeArr = function (arr, obj) {
        var res = [5, obj.size()];
        arr.push(res);
        this.ts(res, obj.id);
        var iterator = obj.iterator();
        var chunk;
        while ((chunk = iterator()))
            this.encodeArrChunk(res, chunk);
    };
    Encoder.prototype.encodeArrChunk = function (arr, chunk) {
        var e_1, _a;
        this.ts(arr, chunk.id);
        if (chunk.del)
            arr.push(chunk.span);
        else {
            var nodes = [];
            var index = this.model.index;
            try {
                for (var _b = tslib_1.__values(chunk.data), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var n = _c.value;
                    this.cNode(nodes, index.get(n));
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            arr.push(nodes);
        }
    };
    Encoder.prototype.encodeStr = function (arr, obj) {
        var res = [4, obj.size()];
        arr.push(res);
        this.ts(res, obj.id);
        var iterator = obj.iterator();
        var chunk;
        while ((chunk = iterator()))
            this.encodeStrChunk(res, chunk);
    };
    Encoder.prototype.encodeStrChunk = function (arr, chunk) {
        this.ts(arr, chunk.id);
        arr.push(chunk.del ? chunk.span : chunk.data);
    };
    Encoder.prototype.encodeBin = function (arr, obj) {
        var res = [6, obj.size()];
        arr.push(res);
        this.ts(res, obj.id);
        var iterator = obj.iterator();
        var chunk;
        while ((chunk = iterator()))
            this.encodeBinChunk(res, chunk);
    };
    Encoder.prototype.encodeBinChunk = function (arr, chunk) {
        this.ts(arr, chunk.id);
        arr.push(chunk.del ? chunk.span : chunk.data);
    };
    Encoder.prototype.cVal = function (arr, obj) {
        var res = [2];
        arr.push(res);
        this.ts(res, obj.id);
        this.cNode(res, obj.node());
    };
    Encoder.prototype.cConst = function (arr, obj) {
        var val = obj.val;
        var res = [1];
        this.ts(res, obj.id);
        if (val !== undefined)
            res.push(val);
        arr.push(res);
    };
    return Encoder;
}());
exports.Encoder = Encoder;
