"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Finder = void 0;
var ArrayApi_1 = require("./ArrayApi");
var ArrayRga_1 = require("../../types/rga-array/ArrayRga");
var BinaryApi_1 = require("./BinaryApi");
var BinaryRga_1 = require("../../types/rga-binary/BinaryRga");
var ObjectApi_1 = require("./ObjectApi");
var ObjectLww_1 = require("../../types/lww-object/ObjectLww");
var json_pointer_1 = require("../../../json-pointer");
var StringApi_1 = require("./StringApi");
var StringRga_1 = require("../../types/rga-string/StringRga");
var ValueApi_1 = require("./ValueApi");
var ValueLww_1 = require("../../types/lww-value/ValueLww");
var Const_1 = require("../../types/const/Const");
var ConstApi_1 = require("./ConstApi");
var Finder = (function () {
    function Finder(node, api) {
        this.node = node;
        this.api = api;
    }
    Finder.prototype.find = function (path) {
        var steps = (0, json_pointer_1.toPath)(path);
        var node = this.node;
        var length = steps.length;
        if (!length)
            return node;
        var i = 0;
        while (i < length && node) {
            var step = steps[i++];
            node = node.container();
            if (!node)
                throw new Error('NOT_CONTAINER');
            if (node instanceof ObjectLww_1.ObjectLww) {
                var nextNode = node.get(String(step));
                if (!nextNode)
                    throw new Error('NOT_FOUND');
                node = nextNode;
            }
            else if (node instanceof ArrayRga_1.ArrayRga) {
                var nextNode = node.getNode(Number(step));
                if (!nextNode)
                    throw new Error('NOT_FOUND');
                node = nextNode;
            }
        }
        return node;
    };
    Finder.prototype.val = function (path) {
        var node = this.find(path);
        if (node instanceof ValueLww_1.ValueLww)
            return new ValueApi_1.ValueApi(this.api, node);
        throw new Error('NOT_VAL');
    };
    Finder.prototype.str = function (path) {
        var node = this.find(path);
        if (node instanceof StringRga_1.StringRga)
            return new StringApi_1.StringApi(this.api, node);
        throw new Error('NOT_STR');
    };
    Finder.prototype.bin = function (path) {
        var node = this.find(path);
        if (node instanceof BinaryRga_1.BinaryRga)
            return new BinaryApi_1.BinaryApi(this.api, node);
        throw new Error('NOT_BIN');
    };
    Finder.prototype.arr = function (path) {
        var node = this.find(path);
        if (node instanceof ArrayRga_1.ArrayRga)
            return new ArrayApi_1.ArrayApi(this.api, node);
        throw new Error('NOT_ARR');
    };
    Finder.prototype.obj = function (path) {
        var obj = this.find(path);
        if (obj instanceof ObjectLww_1.ObjectLww)
            return new ObjectApi_1.ObjectApi(this.api, obj);
        throw new Error('NOT_OBJ');
    };
    Finder.prototype.const = function (path) {
        var obj = this.find(path);
        if (obj instanceof Const_1.Const)
            return new ConstApi_1.ConstApi(obj);
        throw new Error('NOT_CONST');
    };
    return Finder;
}());
exports.Finder = Finder;
