"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelApi = void 0;
var Finder_1 = require("./Finder");
var PatchBuilder_1 = require("../../../json-crdt-patch/PatchBuilder");
var ModelApi = (function () {
    function ModelApi(model) {
        this.model = model;
        this.next = 0;
        this.builder = new PatchBuilder_1.PatchBuilder(this.model.clock);
    }
    ModelApi.prototype.find = function (path) {
        var finder = new Finder_1.Finder(this.getRoot(), this);
        return path ? finder.find(path) : finder;
    };
    ModelApi.prototype.val = function (path) {
        return this.find().val(path);
    };
    ModelApi.prototype.str = function (path) {
        return this.find().str(path);
    };
    ModelApi.prototype.bin = function (path) {
        return this.find().bin(path);
    };
    ModelApi.prototype.arr = function (path) {
        return this.find().arr(path);
    };
    ModelApi.prototype.obj = function (path) {
        return this.find().obj(path);
    };
    ModelApi.prototype.const = function (path) {
        return this.find().const(path);
    };
    ModelApi.prototype.root = function (json) {
        var builder = this.builder;
        builder.root(builder.json(json));
        this.apply();
        return this;
    };
    ModelApi.prototype.getRoot = function () {
        return this.model.root.node();
    };
    ModelApi.prototype.apply = function () {
        var ops = this.builder.patch.ops;
        var length = ops.length;
        for (var i = this.next; i < length; i++)
            this.model.applyOperation(ops[i]);
        this.next = length;
    };
    ModelApi.prototype.advance = function () {
        this.next = this.builder.patch.ops.length;
    };
    ModelApi.prototype.view = function () {
        return this.model.view();
    };
    ModelApi.prototype.flush = function () {
        var patch = this.builder.flush();
        this.next = 0;
        return patch;
    };
    return ModelApi;
}());
exports.ModelApi = ModelApi;
