"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PartialEditFactory = exports.PartialEdit = void 0;
var ClockTable_1 = require("../../json-crdt-patch/codec/clock/ClockTable");
var PartialEditModel_1 = require("./PartialEditModel");
var PartialEdit = (function () {
    function PartialEdit(decoder, encoder, clockTable) {
        if (clockTable === void 0) { clockTable = new ClockTable_1.ClockTable(); }
        this.decoder = decoder;
        this.encoder = encoder;
        this.clockTable = clockTable;
        this.loadList = new Set();
        this.doc = null;
    }
    PartialEdit.prototype.populateLoadList = function (patch) {
        var ops = patch.ops;
        var length = ops.length;
        var loadList = this.loadList;
        for (var i = 0; i < length; i++) {
            var op = ops[i];
            var obj = op.obj;
            if (obj) {
                if (obj.sid === 0) {
                    loadList.add('r');
                    continue;
                }
                var entry = this.clockTable.bySid.get(obj.sid);
                if (!entry)
                    continue;
                var fieldName = "".concat(entry.index.toString(36), "_").concat(obj.time.toString(36));
                loadList.add(fieldName);
            }
        }
    };
    PartialEdit.prototype.getFieldsToLoad = function () {
        return this.loadList;
    };
    PartialEdit.prototype.loadPartialModel = function (fields) {
        this.doc = this.decoder.decodeFields(this.clockTable, fields, PartialEditModel_1.PartialEditModel);
        return this.doc;
    };
    PartialEdit.prototype.applyPatch = function (patch) {
        this.doc.applyPatch(patch);
    };
    PartialEdit.prototype.populateClockTable = function () {
        var doc = this.doc;
        var peers = doc.clock.peers;
        var clockTable = this.clockTable;
        peers.forEach(function (clock, sid) {
            if (!clockTable.bySid.has(sid)) {
                clockTable.push(clock);
            }
        });
    };
    PartialEdit.prototype.getFieldEdits = function () {
        var doc = this.doc;
        var updates = this.encoder.encode(doc, this.clockTable);
        var clockTable = this.clockTable;
        var deletedIds = doc.deletes;
        var length = deletedIds.length;
        var deletes = new Set();
        for (var i = 0; i < length; i++) {
            var id = deletedIds[i];
            var field = "".concat(clockTable.getBySid(id.sid).index, "_").concat(id.time.toString(36));
            deletes.add(field);
        }
        return {
            updates: updates,
            deletes: deletes,
        };
    };
    return PartialEdit;
}());
exports.PartialEdit = PartialEdit;
var PartialEditFactory = (function () {
    function PartialEditFactory(decoder, encoder) {
        this.decoder = decoder;
        this.encoder = encoder;
    }
    PartialEditFactory.prototype.startPartialEdit = function (clockBlob) {
        var reader = this.decoder.dec.reader;
        reader.reset(clockBlob);
        var clockTable = ClockTable_1.ClockTable.decode(reader);
        var partialEdit = new PartialEdit(this.decoder, this.encoder, clockTable);
        return partialEdit;
    };
    return PartialEditFactory;
}());
exports.PartialEditFactory = PartialEditFactory;
