"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectLww = void 0;
var tslib_1 = require("tslib");
var clock_1 = require("../../../json-crdt-patch/clock");
var ObjectLww = (function () {
    function ObjectLww(doc, id) {
        this.doc = doc;
        this.id = id;
        this.latest = new Map();
        this._view = {};
    }
    ObjectLww.prototype.get = function (key) {
        var id = this.latest.get(key);
        if (!id)
            return undefined;
        return this.doc.index.get(id);
    };
    ObjectLww.prototype.getId = function (key) {
        return this.latest.get(key);
    };
    ObjectLww.prototype.put = function (key, id) {
        var currentId = this.latest.get(key);
        if (currentId && (0, clock_1.compare)(currentId, id) >= 0)
            return;
        this.latest.set(key, id);
        return currentId;
    };
    ObjectLww.prototype.view = function () {
        var e_1, _a;
        var _toJson = this._view;
        var obj = {};
        var useCache = true;
        var index = this.doc.index;
        try {
            for (var _b = tslib_1.__values(this.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], id = _d[1];
                var value = index.get(id).view();
                if (value !== undefined) {
                    if (_toJson[key] !== value)
                        useCache = false;
                    obj[key] = value;
                }
                else {
                    if (_toJson[key] !== undefined)
                        useCache = false;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return useCache ? _toJson : (this._view = obj);
    };
    ObjectLww.prototype.toString = function (tab) {
        var e_2, _a;
        if (tab === void 0) { tab = ''; }
        var str = "".concat(this.constructor.name, " ").concat((0, clock_1.toDisplayString)(this.id));
        var length = this.latest.size;
        var cnt = 0;
        try {
            for (var _b = tslib_1.__values(this.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], id = _d[1];
                cnt++;
                var isLast = cnt === length;
                var leftTab = isLast ? '  ' : '│ ';
                str += "\n".concat(tab).concat(isLast ? '└─' : '├─', " ").concat(JSON.stringify(key));
                str += "\n".concat(tab).concat(leftTab, "  \u2514\u2500 ").concat(this.doc.index.get(id).toString(tab + leftTab + '     '));
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return str;
    };
    ObjectLww.prototype.children = function (callback) {
        var index = this.doc.index;
        this.latest.forEach(function (id, key) { return callback(index.get(id)); });
    };
    ObjectLww.prototype.nodes = function (callback) {
        var index = this.doc.index;
        this.latest.forEach(function (id, key) { return callback(index.get(id), key); });
    };
    ObjectLww.prototype.container = function () {
        return this;
    };
    return ObjectLww;
}());
exports.ObjectLww = ObjectLww;
