"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.chunk = exports.normalize = exports.trim = exports.isDeleteComponent = exports.componentLength = exports.append = void 0;
var append = function (op, component) {
    if (!component)
        return;
    if (!op.length) {
        op.push(component);
        return;
    }
    var lastIndex = op.length - 1;
    var last = op[lastIndex];
    switch (typeof component) {
        case 'number': {
            if (typeof last === 'number') {
                if (component > 0 && last > 0)
                    op[lastIndex] = last + component;
                else if (component < 0 && last < 0)
                    op[lastIndex] = last + component;
                else
                    op.push(component);
            }
            else
                op.push(component);
            break;
        }
        case 'string': {
            if (typeof last === 'string')
                op[lastIndex] = last + component;
            else
                op.push(component);
            break;
        }
        case 'object': {
            if (last instanceof Array)
                last[0] = last + component[0];
            else
                op.push(component);
            break;
        }
    }
};
exports.append = append;
var componentLength = function (component) {
    switch (typeof component) {
        case 'number':
            return Math.abs(component);
        case 'string':
            return component.length;
        default:
            return component[0].length;
    }
};
exports.componentLength = componentLength;
var isDeleteComponent = function (component) {
    switch (typeof component) {
        case 'number':
            return component < 0;
        case 'object':
            return true;
        default:
            return false;
    }
};
exports.isDeleteComponent = isDeleteComponent;
var trim = function (op) {
    if (!op.length)
        return;
    var last = op[op.length - 1];
    var isLastRetain = typeof last === 'number' && last > 0;
    if (isLastRetain)
        op.pop();
};
exports.trim = trim;
var normalize = function (op) {
    var op2 = [];
    var length = op.length;
    for (var i = 0; i < length; i++)
        (0, exports.append)(op2, op[i]);
    (0, exports.trim)(op2);
    return op2;
};
exports.normalize = normalize;
var chunk = function (component, offset, maxLength) {
    switch (typeof component) {
        case 'number': {
            return component > 0 ? Math.min(component - offset, maxLength) : -Math.min(-component - offset, maxLength);
        }
        case 'string': {
            var end = Math.min(offset + maxLength, component.length);
            return component.substring(offset, end);
        }
        case 'object': {
            var str = component[0];
            var end = Math.min(offset + maxLength, str.length);
            return [str.substring(offset, end)];
        }
    }
};
exports.chunk = chunk;
