"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IonEncoderFast = void 0;
var Writer_1 = require("../../util/buffers/Writer");
var isSafeInteger = Number.isSafeInteger;
var IonEncoderFast = (function () {
    function IonEncoderFast(writer) {
        if (writer === void 0) { writer = new Writer_1.Writer(); }
        this.writer = writer;
    }
    IonEncoderFast.prototype.encode = function (value) {
        this.writer.reset();
        this.writeIvm();
        this.writeAny(value);
        return this.writer.flush();
    };
    IonEncoderFast.prototype.writeAny = function (value) {
        switch (value) {
            case null:
                return this.writer.u8(0 + 15);
        }
        switch (typeof value) {
            case 'number':
                return this.encodeNumber(value);
            case 'boolean':
                return this.writeBool(value);
        }
    };
    IonEncoderFast.prototype.writeIvm = function () {
        this.writer.u32(0xe00100ea);
    };
    IonEncoderFast.prototype.writeBool = function (bool) {
        this.writer.u8(16 + (bool ? 1 : 0));
    };
    IonEncoderFast.prototype.encodeNumber = function (num) {
        if (isSafeInteger(num))
            this.encodeInteger(num);
        else
            this.writeFloat(num);
    };
    IonEncoderFast.prototype.encodeInteger = function (int) {
        if (int >= 0)
            this.encodeUint(int);
        else
            this.encodeNint(int);
    };
    IonEncoderFast.prototype.encodeUint = function (uint) {
        if (!uint)
            this.writer.u8(32);
        else if (uint <= 0xff)
            this.writer.u16(((32 + 1) << 8) + uint);
        else if (uint <= 0xffff)
            this.writer.u8u16(32 + 2, uint);
        else if (uint <= 0xffffff)
            this.writer.u32(((32 + 3) << 24) + uint);
        else if (uint <= 0xffffffff)
            this.writer.u8u32(32 + 4, uint);
        else {
            var lo = uint | 0;
            if (lo < 0)
                lo += 4294967296;
            var hi = uint - lo;
            hi /= 4294967296;
            if (uint <= 0xffffffffff) {
                this.writer.u16(((32 + 5) << 8) + hi);
                this.writer.u32(lo);
            }
            else if (uint <= 0xffffffffffff) {
                this.writer.u8u16(32 + 6, hi);
                this.writer.u32(lo);
            }
            else {
                this.writer.u16(((32 + 7) << 8) + (hi >> 16));
                this.writer.u16(hi & 0xffff);
                this.writer.u32(lo);
            }
        }
    };
    IonEncoderFast.prototype.encodeNint = function (int) {
        var uint = -int;
        if (uint <= 0xff)
            this.writer.u16(((48 + 1) << 8) + uint);
        else if (uint <= 0xffff)
            this.writer.u8u16(48 + 2, uint);
        else if (uint <= 0xffffff)
            this.writer.u32(((48 + 3) << 24) + uint);
        else if (uint <= 0xffffffff)
            this.writer.u8u32(48 + 4, uint);
        else {
            var lo = uint | 0;
            if (lo < 0)
                lo += 4294967296;
            var hi = uint - lo;
            hi /= 4294967296;
            if (uint <= 0xffffffffff) {
                this.writer.u16(((48 + 5) << 8) + hi);
                this.writer.u32(lo);
            }
            else if (uint <= 0xffffffffffff) {
                this.writer.u8u16(48 + 6, hi);
                this.writer.u32(lo);
            }
            else {
                this.writer.u16(((48 + 7) << 8) + (hi >> 16));
                this.writer.u16(hi & 0xffff);
                this.writer.u32(lo);
            }
        }
    };
    IonEncoderFast.prototype.writeFloat = function (float) {
        this.writer.u8f64(64 + 8, float);
    };
    return IonEncoderFast;
}());
exports.IonEncoderFast = IonEncoderFast;
