"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromBase64 = exports.createFromBase64 = void 0;
var bufferToUint8Array_1 = require("../buffers/bufferToUint8Array");
var constants_1 = require("./constants");
var E = '=';
var createFromBase64 = function (chars) {
    if (chars === void 0) { chars = constants_1.alphabet; }
    if (chars.length !== 64)
        throw new Error('chars must be 64 characters long');
    var table = [];
    for (var i = 0; i < chars.length; i++)
        table[chars.charCodeAt(i)] = i;
    return function (encoded) {
        if (!encoded)
            return new Uint8Array(0);
        var length = encoded.length;
        if (length % 4 !== 0)
            throw new Error('Base64 string length must be a multiple of 4');
        var mainLength = encoded[length - 1] !== E ? length : length - 4;
        var bufferLength = (length / 4) * 3;
        var padding = 0;
        if (encoded[length - 2] === E) {
            padding = 2;
            bufferLength -= 2;
        }
        else if (encoded[length - 1] === E) {
            padding = 1;
            bufferLength -= 1;
        }
        var buf = new Uint8Array(bufferLength);
        var j = 0;
        var i = 0;
        for (; i < mainLength; i += 4) {
            var sextet0 = table[encoded.charCodeAt(i)];
            var sextet1 = table[encoded.charCodeAt(i + 1)];
            var sextet2 = table[encoded.charCodeAt(i + 2)];
            var sextet3 = table[encoded.charCodeAt(i + 3)];
            if (sextet0 === undefined || sextet1 === undefined || sextet2 === undefined || sextet3 === undefined)
                throw new Error('INVALID_BASE64_STRING');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
            buf[j + 1] = (sextet1 << 4) | (sextet2 >> 2);
            buf[j + 2] = (sextet2 << 6) | sextet3;
            j += 3;
        }
        if (padding === 2) {
            var sextet0 = table[encoded.charCodeAt(mainLength)];
            var sextet1 = table[encoded.charCodeAt(mainLength + 1)];
            if (sextet0 === undefined || sextet1 === undefined)
                throw new Error('INVALID_BASE64_STRING');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
        }
        else if (padding === 1) {
            var sextet0 = table[encoded.charCodeAt(mainLength)];
            var sextet1 = table[encoded.charCodeAt(mainLength + 1)];
            var sextet2 = table[encoded.charCodeAt(mainLength + 2)];
            if (sextet0 === undefined || sextet1 === undefined || sextet2 === undefined)
                throw new Error('INVALID_BASE64_STRING');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
            buf[j + 1] = (sextet1 << 4) | (sextet2 >> 2);
        }
        return buf;
    };
};
exports.createFromBase64 = createFromBase64;
var fromBase64Cpp = constants_1.hasBuffer ? function (encoded) { return (0, bufferToUint8Array_1.bufferToUint8Array)(Buffer.from(encoded, 'base64')); } : null;
var fromBase64Native = (0, exports.createFromBase64)();
exports.fromBase64 = !fromBase64Cpp
    ? fromBase64Native
    : function (encoded) { return (encoded.length > 48 ? fromBase64Cpp(encoded) : fromBase64Native(encoded)); };
