"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.encode = exports.encodeTe = exports.encodeNative = exports.encodeFrom = exports.encodeUtf8Write = void 0;
var hasBuffer = typeof Buffer !== undefined;
var utf8Write = hasBuffer
    ? Buffer.prototype.utf8Write
    : null;
var from = hasBuffer ? Buffer.from : null;
var encodeUtf8Write = function (arr, str, pos, maxLength) {
    return utf8Write.call(arr, str, pos, maxLength);
};
exports.encodeUtf8Write = encodeUtf8Write;
var encodeFrom = function (arr, str, pos, maxLength) {
    var offset = arr.byteOffset + pos;
    var buf = from(arr.buffer).subarray(offset, offset + maxLength);
    return buf.write(str, 0, maxLength, 'utf8');
};
exports.encodeFrom = encodeFrom;
var encodeNative = function (arr, str, pos, maxLength) {
    var length = str.length;
    var start = pos;
    var curr = 0;
    while (curr < length) {
        var value = str.charCodeAt(curr++);
        if ((value & 0xffffff80) === 0) {
            arr[pos++] = value;
            continue;
        }
        else if ((value & 0xfffff800) === 0) {
            arr[pos++] = ((value >> 6) & 0x1f) | 0xc0;
        }
        else {
            if (value >= 0xd800 && value <= 0xdbff) {
                if (curr < length) {
                    var extra = str.charCodeAt(curr);
                    if ((extra & 0xfc00) === 0xdc00) {
                        curr++;
                        value = ((value & 0x3ff) << 10) + (extra & 0x3ff) + 0x10000;
                    }
                }
            }
            if ((value & 0xffff0000) === 0) {
                arr[pos++] = ((value >> 12) & 0x0f) | 0xe0;
                arr[pos++] = ((value >> 6) & 0x3f) | 0x80;
            }
            else {
                arr[pos++] = ((value >> 18) & 0x07) | 0xf0;
                arr[pos++] = ((value >> 12) & 0x3f) | 0x80;
                arr[pos++] = ((value >> 6) & 0x3f) | 0x80;
            }
        }
        arr[pos++] = (value & 0x3f) | 0x80;
    }
    return pos - start;
};
exports.encodeNative = encodeNative;
var textEncoder = typeof TextEncoder !== 'undefined' ? new TextEncoder() : null;
var encodeTe = function (arr, str, pos, maxLength) {
    return textEncoder.encodeInto(str, arr.subarray(pos, pos + maxLength)).written;
};
exports.encodeTe = encodeTe;
exports.encode = utf8Write ? exports.encodeUtf8Write : from ? exports.encodeFrom : exports.encodeNative;
