"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rlSplay = exports.lrSplay = exports.llSplay = exports.rrSplay = exports.lSplay = exports.rSplay = exports.splay = exports.remove = exports.insert = exports.insertLeft = exports.insertRight = exports.findOrNextLower = exports.find = void 0;
var find = function (root, key, comparator) {
    var curr = root;
    while (curr) {
        var cmp = comparator(key, curr.k);
        if (cmp === 0)
            return curr;
        curr = cmp < 0 ? curr.l : curr.r;
    }
    return curr;
};
exports.find = find;
var findOrNextLower = function (root, key, comparator) {
    var curr = root;
    var result = undefined;
    while (curr) {
        var cmp = comparator(curr.k, key);
        if (cmp === 0)
            return curr;
        if (cmp > 0)
            curr = curr.l;
        else {
            var next = curr.r;
            result = curr;
            if (!next)
                return result;
            curr = next;
        }
    }
    return result;
};
exports.findOrNextLower = findOrNextLower;
var insertRight = function (node, p) {
    var r = (node.r = p.r);
    p.r = node;
    node.p = p;
    if (r)
        r.p = node;
};
exports.insertRight = insertRight;
var insertLeft = function (node, p) {
    var l = (node.l = p.l);
    p.l = node;
    node.p = p;
    if (l)
        l.p = node;
};
exports.insertLeft = insertLeft;
var insert = function (root, node, comparator) {
    if (!root)
        return node;
    var key = node.k;
    var curr = root;
    while (curr) {
        var cmp = comparator(key, curr.k);
        var next = cmp < 0 ? curr.l : curr.r;
        if (!next) {
            if (cmp < 0)
                (0, exports.insertLeft)(node, curr);
            else
                (0, exports.insertRight)(node, curr);
            break;
        }
        else
            curr = next;
    }
    return root;
};
exports.insert = insert;
var remove = function (root, node) {
    var p = node.p;
    var l = node.l;
    var r = node.r;
    node.p = node.l = node.r = undefined;
    if (!l && !r) {
        if (!p)
            return undefined;
        else if (p.l === node)
            p.l = undefined;
        else
            p.r = undefined;
        return root;
    }
    else if (l && r) {
        var mostRightChildFromLeft = l;
        while (mostRightChildFromLeft.r)
            mostRightChildFromLeft = mostRightChildFromLeft.r;
        mostRightChildFromLeft.r = r;
        r.p = mostRightChildFromLeft;
        if (!p) {
            l.p = undefined;
            return l;
        }
        if (p.l === node)
            p.l = l;
        else
            p.r = l;
        l.p = p;
        return root;
    }
    var child = (l || r);
    child.p = p;
    if (!p)
        return child;
    else if (p.l === node)
        p.l = child;
    else
        p.r = child;
    return root;
};
exports.remove = remove;
var splay = function (root, node, repeat) {
    var p = node.p;
    if (!p)
        return root;
    var pp = p.p;
    var l2 = p.l === node;
    if (!pp)
        return l2 ? (0, exports.rSplay)(node, p) : (0, exports.lSplay)(node, p);
    var l1 = pp.l === p;
    if (l1 && l2)
        root = (0, exports.llSplay)(root, node, p, pp);
    else if (!l1 && !l2)
        root = (0, exports.rrSplay)(root, node, p, pp);
    else if (l1 && !l2)
        root = (0, exports.lrSplay)(root, node, p, pp);
    else
        root = (0, exports.rlSplay)(root, node, p, pp);
    if (repeat > 1)
        return (0, exports.splay)(root, node, repeat - 1);
    return root;
};
exports.splay = splay;
var rSplay = function (c2, c1) {
    var b = c2.r;
    c2.p = undefined;
    c2.r = c1;
    c1.p = c2;
    c1.l = b;
    if (b)
        b.p = c1;
    return c2;
};
exports.rSplay = rSplay;
var lSplay = function (c2, c1) {
    var b = c2.l;
    c2.p = undefined;
    c2.l = c1;
    c1.p = c2;
    c1.r = b;
    if (b)
        b.p = c1;
    return c2;
};
exports.lSplay = lSplay;
var rrSplay = function (root, c3, c2, c1) {
    var a = c1.l;
    var b = c2.l;
    var c = c3.l;
    var p = c1.p;
    c3.p = p;
    c3.l = c2;
    c2.p = c3;
    c2.l = c1;
    c2.r = c;
    c1.p = c2;
    c1.l = a;
    c1.r = b;
    if (a)
        a.p = c1;
    if (b)
        b.p = c1;
    if (c)
        c.p = c2;
    if (!p)
        root = c3;
    else if (p.l === c1)
        p.l = c3;
    else
        p.r = c3;
    return root;
};
exports.rrSplay = rrSplay;
var llSplay = function (root, c3, c2, c1) {
    var a = c1.r;
    var b = c2.r;
    var c = c3.r;
    var p = c1.p;
    c3.p = p;
    c3.r = c2;
    c2.p = c3;
    c2.l = c;
    c2.r = c1;
    c1.p = c2;
    c1.l = b;
    c1.r = a;
    if (a)
        a.p = c1;
    if (b)
        b.p = c1;
    if (c)
        c.p = c2;
    if (!p)
        root = c3;
    else if (p.l === c1)
        p.l = c3;
    else
        p.r = c3;
    return root;
};
exports.llSplay = llSplay;
var lrSplay = function (root, c3, c2, c1) {
    var a = c1.r;
    var b = c2.l;
    var c = c3.l;
    var d = c3.r;
    var p = c1.p;
    c3.p = p;
    c3.l = c2;
    c3.r = c1;
    c2.p = c3;
    c2.l = b;
    c2.r = c;
    c1.p = c3;
    c1.l = d;
    c1.r = a;
    if (a)
        a.p = c1;
    if (b)
        b.p = c2;
    if (c)
        c.p = c2;
    if (d)
        d.p = c1;
    if (!p)
        root = c3;
    else if (p.l === c1)
        p.l = c3;
    else
        p.r = c3;
    return root;
};
exports.lrSplay = lrSplay;
var rlSplay = function (root, c3, c2, c1) {
    var a = c1.l;
    var b = c2.r;
    var c = c3.r;
    var d = c3.l;
    var p = c1.p;
    c3.p = p;
    c3.l = c1;
    c3.r = c2;
    c2.p = c3;
    c2.l = c;
    c2.r = b;
    c1.p = c3;
    c1.l = a;
    c1.r = d;
    if (a)
        a.p = c1;
    if (b)
        b.p = c2;
    if (c)
        c.p = c2;
    if (d)
        d.p = c1;
    if (!p)
        root = c3;
    else if (p.l === c1)
        p.l = c3;
    else
        p.r = c3;
    return root;
};
exports.rlSplay = rlSplay;
