"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkerPool = void 0;
var tslib_1 = require("tslib");
var Defer_1 = require("../Defer");
var worker_threads_1 = require("worker_threads");
var WorkerPoolModule_1 = require("./WorkerPoolModule");
var path = tslib_1.__importStar(require("path"));
var WorkerPool = (function () {
    function WorkerPool() {
        this.seq = 0;
        this.nextWorker = 0;
        this.workers = [];
        this.requests = new Map();
        this.modules = new Map();
    }
    WorkerPool.prototype.queue = function () {
        return this.requests.size;
    };
    WorkerPool.prototype.addThread = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var fileName, worker, _a, _b, module_1, e_1_1;
            var e_1, _c;
            var _this = this;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        fileName = path.join(__dirname, 'worker', 'main');
                        worker = new worker_threads_1.Worker(fileName);
                        return [4, new Promise(function (resolve) {
                                worker.once('message', function (msg) { return resolve(); });
                            })];
                    case 1:
                        _d.sent();
                        worker.on('message', function (msg) {
                            if (msg instanceof Array)
                                _this.onResponse(msg);
                        });
                        _d.label = 2;
                    case 2:
                        _d.trys.push([2, 7, 8, 9]);
                        _a = tslib_1.__values(this.modules.values()), _b = _a.next();
                        _d.label = 3;
                    case 3:
                        if (!!_b.done) return [3, 6];
                        module_1 = _b.value;
                        return [4, module_1.setupWorker(worker)];
                    case 4:
                        _d.sent();
                        _d.label = 5;
                    case 5:
                        _b = _a.next();
                        return [3, 3];
                    case 6: return [3, 9];
                    case 7:
                        e_1_1 = _d.sent();
                        e_1 = { error: e_1_1 };
                        return [3, 9];
                    case 8:
                        try {
                            if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7];
                    case 9:
                        this.workers.push(worker);
                        return [2];
                }
            });
        });
    };
    WorkerPool.prototype.addThreads = function (count) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var promises, i;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        promises = [];
                        for (i = 0; i < count; i++)
                            promises.push(this.addThread());
                        return [4, Promise.all(promises)];
                    case 1:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    WorkerPool.prototype.load = function (name, file) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var module;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.modules.has(name))
                            throw new Error("Duplicate [module = ".concat(name, "]."));
                        module = new WorkerPoolModule_1.WorkerPoolModule(this, name, file);
                        this.modules.set(name, module);
                        return [4, Promise.all(this.workers.map(function (worker) { return module.setupWorker(worker); }))];
                    case 1:
                        _a.sent();
                        return [2, module];
                }
            });
        });
    };
    WorkerPool.prototype.exec = function (id, data, transferList, worker) {
        if (worker === void 0) { worker = this.pickWorker(); }
        var seq = this.seq++;
        var future = new Defer_1.Defer();
        var request = [seq, id, data];
        this.requests.set(seq, future);
        try {
            worker.postMessage(request, transferList);
            return future.promise;
        }
        catch (_a) {
            this.requests.delete(seq);
            return Promise.reject(new Error('PORT'));
        }
    };
    WorkerPool.prototype.onResponse = function (_a) {
        var _b = tslib_1.__read(_a, 3), seq = _b[0], data = _b[1], isError = _b[2];
        var requests = this.requests;
        var future = requests.get(seq);
        if (!future)
            return;
        requests.delete(seq);
        if (isError)
            future.reject(data);
        else
            future.resolve(data);
    };
    WorkerPool.prototype.pickWorker = function () {
        var worker = this.workers[this.nextWorker];
        this.nextWorker = (this.nextWorker + 1) % this.workers.length;
        return worker;
    };
    return WorkerPool;
}());
exports.WorkerPool = WorkerPool;
