"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkerPoolModule = void 0;
var tslib_1 = require("tslib");
var WorkerResponse_1 = require("./WorkerResponse");
var WorkerPoolModule = (function () {
    function WorkerPoolModule(pool, name, file) {
        this.pool = pool;
        this.name = name;
        this.file = file;
        this.toId = new Map();
        this.type = new Map();
    }
    WorkerPoolModule.prototype.setupWorker = function (worker) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var msg, list, list_1, list_1_1, _a, id, name_1, type;
            var e_1, _b;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        msg = {
                            type: 'load',
                            file: this.file,
                        };
                        worker.postMessage(msg);
                        return [4, new Promise(function (resolve) {
                                var listener = function (msg) {
                                    if (typeof msg === 'object' && msg.type === 'loaded') {
                                        worker.removeListener('message', listener);
                                        resolve(msg);
                                    }
                                };
                                worker.addListener('message', listener);
                            })];
                    case 1:
                        list = (_c.sent()).list;
                        try {
                            for (list_1 = tslib_1.__values(list), list_1_1 = list_1.next(); !list_1_1.done; list_1_1 = list_1.next()) {
                                _a = tslib_1.__read(list_1_1.value, 3), id = _a[0], name_1 = _a[1], type = _a[2];
                                this.toId.set(name_1, id);
                                this.type.set(name_1, type === 'ch' ? 1 : 0);
                            }
                        }
                        catch (e_1_1) { e_1 = { error: e_1_1 }; }
                        finally {
                            try {
                                if (list_1_1 && !list_1_1.done && (_b = list_1.return)) _b.call(list_1);
                            }
                            finally { if (e_1) throw e_1.error; }
                        }
                        return [2];
                }
            });
        });
    };
    WorkerPoolModule.prototype.fn = function (name, worker) {
        var _this = this;
        var id = this.toId.get(name);
        if (id === undefined)
            throw new Error('UNKNOWN_FN');
        return (function (req, transferList) { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, this.pool.exec(id, req, transferList, worker)];
                case 1: return [2, _a.sent()];
            }
        }); }); });
    };
    WorkerPoolModule.prototype.ch = function (name, worker) {
        var _this = this;
        var id = this.toId.get(name);
        if (id === undefined)
            throw new Error('UNKNOWN_CH');
        return (function (req, transferList) {
            worker = worker || _this.pool.pickWorker();
            var cb;
            var closed = false;
            var seq = _this.pool.seq;
            var response = _this.pool.exec(id, req, transferList, worker);
            var take = function (callback) {
                if (closed)
                    throw new Error('CHANNEL_CLOSED');
                cb = callback;
            };
            var send = function (data) {
                if (closed)
                    throw new Error('CHANNEL_CLOSED');
                var transferList;
                if (data instanceof WorkerResponse_1.WorkerResponse) {
                    transferList = data.transferList;
                    data = data.data;
                }
                var msg = [[seq], data];
                worker.postMessage(msg, transferList);
            };
            var listener = function (msg) {
                if (msg instanceof Array) {
                    if (msg[0] instanceof Array) {
                        if (msg[0][0] === seq) {
                            if (cb)
                                cb(msg[1]);
                        }
                    }
                }
            };
            worker.addListener('message', listener);
            return [
                take,
                send,
                response.finally(function () {
                    closed = true;
                    worker.removeListener('message', listener);
                }),
            ];
        });
    };
    WorkerPoolModule.prototype.api = function (worker) {
        var e_2, _a;
        var methods = {};
        try {
            for (var _b = tslib_1.__values(this.toId.keys()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var name_2 = _c.value;
                var isChannel = this.type.get(name_2) === 1;
                methods[name_2] = isChannel ? this.ch(name_2, worker) : this.fn(name_2, worker);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return methods;
    };
    return WorkerPoolModule;
}());
exports.WorkerPoolModule = WorkerPoolModule;
