"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkerRuntime = void 0;
var tslib_1 = require("tslib");
var WorkerResponse_1 = require("../WorkerResponse");
var WorkerChannel_1 = require("../WorkerChannel");
var WorkerRuntime = (function () {
    function WorkerRuntime(port) {
        var _this = this;
        this.port = port;
        this.fn = [];
        this.channels = new Map();
        this.port.on('message', function (msg) {
            if (msg instanceof Array) {
                if (msg[0] instanceof Array)
                    _this.onChannelMsg(msg);
                else
                    _this.onRequest(msg);
            }
            else {
                switch (msg.type) {
                    case 'load':
                        _this.onLoad(msg);
                }
            }
        });
    }
    WorkerRuntime.prototype.onChannelMsg = function (msg) {
        var _a = tslib_1.__read(msg, 2), _b = tslib_1.__read(_a[0], 1), seq = _b[0], data = _a[1];
        var callback = this.channels.get(seq);
        if (callback)
            callback(data);
    };
    WorkerRuntime.prototype.onRequest = function (_a) {
        var _b = tslib_1.__read(_a, 3), seq = _b[0], method = _b[1], data = _b[2];
        var fn = this.fn[method];
        if (!fn) {
            return;
        }
        if (fn instanceof WorkerChannel_1.WorkerChannel)
            this.onCh(seq, fn, data);
        else
            this.onFn(seq, fn, data);
    };
    WorkerRuntime.prototype.onFn = function (seq, fn, data) {
        var _this = this;
        try {
            var result = fn(data);
            if (result instanceof Promise) {
                result
                    .then(function (result) {
                    _this.sendResponse(seq, result);
                })
                    .catch(function (error) {
                    _this.sendError(seq, error);
                });
            }
            else {
                this.sendResponse(seq, result);
            }
        }
        catch (error) {
            this.sendError(seq, error);
        }
    };
    WorkerRuntime.prototype.onCh = function (seq, fn, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var closed, send, take, response, error_1;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        closed = false;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        send = function (data) {
                            if (closed)
                                throw new Error('CHANNEL_CLOSED');
                            var transferList;
                            if (data instanceof WorkerResponse_1.WorkerResponse) {
                                transferList = data.transferList;
                                data = data.data;
                            }
                            var msg = [[seq], data];
                            _this.port.postMessage(msg, transferList);
                        };
                        take = function (callback) {
                            if (closed)
                                throw new Error('CHANNEL_CLOSED');
                            _this.channels.set(seq, callback);
                        };
                        return [4, fn.fn(data, take, send)];
                    case 2:
                        response = _a.sent();
                        closed = true;
                        this.channels.delete(seq);
                        this.sendResponse(seq, response);
                        return [3, 4];
                    case 3:
                        error_1 = _a.sent();
                        closed = true;
                        this.channels.delete(seq);
                        this.sendError(seq, error_1);
                        return [3, 4];
                    case 4: return [2];
                }
            });
        });
    };
    WorkerRuntime.prototype.sendResponse = function (seq, response) {
        var transferList;
        if (response instanceof WorkerResponse_1.WorkerResponse) {
            transferList = response.transferList;
            response = response.data;
        }
        var msg = [seq, response];
        this.port.postMessage(msg, transferList);
    };
    WorkerRuntime.prototype.sendError = function (seq, error) {
        try {
            var response = [seq, error, true];
            this.port.postMessage(response);
        }
        catch (_a) {
            var response = [seq, 0, true];
            this.port.postMessage(response);
        }
    };
    WorkerRuntime.prototype.onLoad = function (_a) {
        var e_1, _b;
        var file = _a.file;
        var methods = require(file).methods;
        var keys = Object.keys(methods).sort();
        var list = [];
        var _loop_1 = function (key) {
            var isChannel = methods[key] instanceof WorkerChannel_1.WorkerChannel;
            var fn = typeof methods[key] === 'function'
                ? methods[key]
                : isChannel
                    ? methods[key]
                    : function () { return new WorkerResponse_1.WorkerResponse(methods[key], []); };
            var id = this_1.fn.length;
            list.push([id, key, isChannel ? 'ch' : 'fn']);
            this_1.fn.push(fn);
        };
        var this_1 = this;
        try {
            for (var keys_1 = tslib_1.__values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                var key = keys_1_1.value;
                _loop_1(key);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (keys_1_1 && !keys_1_1.done && (_b = keys_1.return)) _b.call(keys_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var response = {
            type: 'loaded',
            list: list,
        };
        this.port.postMessage(response);
    };
    WorkerRuntime.prototype.ready = function () {
        var msg = { type: 'ready' };
        this.port.postMessage(msg);
    };
    return WorkerRuntime;
}());
exports.WorkerRuntime = WorkerRuntime;
